from rpython.annotator.model import SomeChar, SomeUnicodeCodePoint
from rpython.rlib.rstring import INIT_SIZE
from rpython.rtyper.lltypesystem import lltype
from rpython.rtyper.rmodel import Repr


class AbstractStringBuilderRepr(Repr):
    def rtyper_new(self, hop):
        if len(hop.args_v) == 0:
            v_arg = hop.inputconst(lltype.Signed, INIT_SIZE)
        else:
            v_arg = hop.inputarg(lltype.Signed, 0)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_new, v_arg)

    def rtype_method_append(self, hop):
        if isinstance(hop.args_s[1], (SomeChar, SomeUnicodeCodePoint)):
            vlist = hop.inputargs(self, self.char_repr)
            func = self.ll_append_char
        else:
            vlist = hop.inputargs(self, self.string_repr)
            func = self.ll_append
        hop.exception_cannot_occur()
        return hop.gendirectcall(func, *vlist)

    def rtype_method_append_slice(self, hop):
        vlist = hop.inputargs(self, self.string_repr,
                              lltype.Signed, lltype.Signed)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_append_slice, *vlist)

    def rtype_method_append_multiple_char(self, hop):
        vlist = hop.inputargs(self, self.char_repr, lltype.Signed)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_append_multiple_char, *vlist)

    def rtype_method_append_charpsize(self, hop):
        vlist = hop.inputargs(self, self.raw_ptr_repr, lltype.Signed)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_append_charpsize, *vlist)

    def rtype_method_getlength(self, hop):
        vlist = hop.inputargs(self)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_getlength, *vlist)

    def rtype_method_build(self, hop):
        vlist = hop.inputargs(self)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_build, *vlist)

    def rtype_bool(self, hop):
        vlist = hop.inputargs(self)
        hop.exception_cannot_occur()
        return hop.gendirectcall(self.ll_bool, *vlist)

    def convert_const(self, value):
        if value is None:
            return self.empty()
        s = value.build()
        ll_obj = self.ll_new(len(s))
        self.ll_append(ll_obj, self.convert_to_ll(s))
        return ll_obj



"""
This file contains an abstract domain IntBound for word-sized integers. This is
used to perform abstract interpretation on traces, specifically the integer
operations on them.

The abstract domain tracks a (signed) upper and lower bound (both ends
inclusive) for every integer variable in the trace. It also tracks which bits
of a range are known 0 or known 1 (the remaining bits are unknown. The ranges
and the known bits feed back into each other, ie we can improve the range if
some upper bits have known values, and we can learn some known bits from the
range too. Every instance of IntBound represents a set of concrete integers.

We do the analysis at the same time as optimization. We initialize all integer
variables to have an unknown range, with no known bits. Then we proceed along
the trace and improve the ranges. We can shrink ranges if we see integer
comparisons followed by guards. We can learn some bits of an integer if there
are bit-operations such as `and` (masking out some bits), followed by a guard.

For every operation in the trace we use a "transfer function" that computes an
IntBound instance for the result of that operation, given the IntBounds of the
arguments. Those functions are called `..._bound`, eg `add_bound`, `and_bound`,
`neg_bound`, etc. Applying the transfer functions while we encounter them along
the trace is forwards reasoning.

We can also reason backwards (but we only do that in a limited way). Here's an
example:

i1 = int_add(i0, 1)
i2 = int_lt(i1, 100)
guard_true(i2)

At the last guard we learn that i1 < 100 must be true, and from that we can
conclude that i0 < 99 in the rest of the trace (this is not quite true due to
possible overflow of the int_add).

More generally, when we shrink (ie make more precise) an IntBound instance due
to a guard, we can often conclude something about earlier variables in the
trace. To reason backwards we look at the operation that created a variable and
then compute the implications.

The reason for having both a range and known bits are that each of them is good
for different situations. Range knownledge is useful for comparisons and
"linear" operations like additions, subtractions, multiplications, etc.
Knowledge about certain bits is good for bit twiddling code, bitfields, stuff
like that.
"""

import sys
from rpython.rlib.rarithmetic import ovfcheck, LONG_BIT, maxint, is_valid_int, r_uint, intmask
from rpython.rlib.objectmodel import we_are_translated, always_inline
from rpython.rtyper.lltypesystem import lltype
from rpython.rtyper.lltypesystem.lloperation import llop
from rpython.jit.metainterp.resoperation import rop, ResOperation
from rpython.jit.metainterp.optimize import InvalidLoop
from rpython.jit.metainterp.optimizeopt.info import AbstractInfo, INFO_NONNULL,\
     INFO_UNKNOWN, INFO_NULL
from rpython.jit.metainterp.history import ConstInt


MAXINT = maxint
MININT = -maxint - 1

IS_64_BIT = sys.maxint > 2**32

TNUM_UNKNOWN = r_uint(0), r_uint(-1)
TNUM_KNOWN_ZERO = r_uint(0), r_uint(0)
TNUM_KNOWN_BITWISEONE = r_uint(-1), r_uint(0)
TNUM_ONLY_VALUE_DEFAULT = r_uint(0)
TNUM_ONLY_MASK_UNKNOWN = r_uint(-1)
TNUM_ONLY_MASK_DEFAULT = TNUM_ONLY_MASK_UNKNOWN


class IntBound(AbstractInfo):
    """
    Abstract domain representation of an integer,
    approximating via integer bounds and known-bits
    tri-state numbers.
    """
    _attrs_ = ('upper', 'lower', 'tvalue', 'tmask')

    def __init__(self, lower=MININT, upper=MAXINT,
                 tvalue=TNUM_ONLY_VALUE_DEFAULT,
                 tmask=TNUM_ONLY_MASK_DEFAULT,
                 do_shrinking=True):
        """
        Instantiates an abstract representation of integer.
        The default parameters set this abstract int to
        contain all integers.

        It is recommended to use the indirect constructors
        below instead of this one.
        """

        self.lower = lower
        self.upper = upper

        # known-bit analysis using tristate/knownbits numbers:
        # every bit can be either 0, 1, or unknown (?).
        # the encoding of these three states is:
        # value       0 1 ?
        # tvalue bit  0 1 0
        # tmask bit   0 0 1
        # the combination tvalue=tmask=1 is forbidden
        assert is_valid_tnum(tvalue, tmask)
        self.tvalue = tvalue
        self.tmask = tmask         # bit=1 means unknown

        # check for unexpected overflows:
        if not we_are_translated():
            assert type(upper) is not long or is_valid_int(upper)
            assert type(lower) is not long or is_valid_int(lower)

        if do_shrinking:
            self.shrink()
            assert self._debug_check()

    @staticmethod
    def from_constant(value):
        """
        Constructs an abstract integer that represents a constant (a completely
        known integer).
        """
        # this one does NOT require a r_uint for `value`.
        assert not isinstance(value, r_uint)
        tvalue = value
        tmask = 0
        bvalue = value
        do_shrinking = False
        if not isinstance(value, int):
            # workaround for AddressAsInt / symbolic ints
            # by CF
            tvalue = 0
            tmask = -1
            bvalue = 0
            do_shrinking = True
        b = IntBound(lower=bvalue,
                     upper=bvalue,
                     tvalue=r_uint(tvalue),
                     tmask=r_uint(tmask), do_shrinking=do_shrinking)
        return b

    @staticmethod
    def unbounded():
        """
        Constructs an abstract integer that is completely unknown (e.g. it
        contains every integer).
        """
        return IntBound(do_shrinking=False)

    @staticmethod
    def nonnegative():
        """
        Construct a non-negative abstract integer.
        """
        return IntBound(lower=0)

    @staticmethod
    def from_knownbits(tvalue, tmask, do_unmask=False):
        """
        Constructs an abstract integer where the bits determined by `tvalue`
        and `tmask` are (un-)known. tvalue and tmask must be r_uints.
        """
        assert isinstance(tvalue, r_uint) and isinstance(tmask, r_uint)
        if do_unmask:
            tvalue = unmask_zero(tvalue, tmask)
        return IntBound(tvalue=tvalue,
                        tmask=tmask)

    # ____________________________________________________________
    # a bunch of slightly artificial methods that are needed to make some of
    # the Z3 proofs in test_z3intbound possible, they are overridden there

    @staticmethod
    @always_inline
    def new(lower, upper, tvalue, tmask):
        """ helper factory to construct a new IntBound. overridden in
        test_z3intbound """
        return IntBound(lower, upper, tvalue, tmask)

    intmask = staticmethod(intmask)
    r_uint = staticmethod(r_uint)

    @staticmethod
    @always_inline
    def _add_check_overflow(a, b, value_if_overflow):
        """ returns a + b, or value_if_overflow if that (signed) addition would
        overflow """
        try:
            return ovfcheck(a + b)
        except OverflowError:
            return value_if_overflow

    @staticmethod
    @always_inline
    def _sub_check_overflow(a, b, value_if_overflow):
        """ returns a - b, or value_if_overflow if that (signed) subtraction would
        overflow """
        try:
            return ovfcheck(a - b)
        except OverflowError:
            return value_if_overflow

    @staticmethod
    @always_inline
    def _urshift(a, b):
        return r_uint(a) >> r_uint(b)

    # ____________________________________________________________

    @staticmethod
    def _to_dec_or_hex_str_heuristics(num):
        # a few formatting heuristics
        if -1000 <= num <= 1000:
            return str(num)
        if num == MININT:
            return "MININT"
        if num == MAXINT:
            return "MAXINT"
        if num >= 0:
            diff = MAXINT - num
            if diff < 1000:
                return "MAXINT - %s" % diff
        else:
            diff = -(MININT - num) # can't overflow because num < 0
            if diff < 1000:
                return "MININT + %s" % diff
        uintnum = r_uint(num)
        if uintnum & (uintnum - 1) == 0:
            # power of two, use hex
            return hex(num)
        # format number as decimal if fewer than 6 significant
        # digits, otherwise use hex
        curr = num
        exp10 = 0
        while curr % 10 == 0:
            curr //= 10
            exp10 += 1
        s = str(num)
        if len(s) - exp10 >= 6:
            return hex(num)
        return s

    def _are_knownbits_implied(self):
        """ return True if the knownbits of self are a direct consequence of
        the range of self (and thus carry no extra information) """
        tvalue, tmask = self._tnum_implied_by_bounds()
        return self.tmask == tmask and self.tvalue == tvalue

    def _are_bounds_implied(self):
        """ return True if the bounds of self are a direct consequence of the
        knownbits of self (and thus carry no extra information) """
        lower = self._get_minimum_signed_by_knownbits()
        upper = self._get_maximum_signed_by_knownbits()
        return self.lower == lower and self.upper == upper

    def __repr__(self):
        if self.is_unbounded():
            return "IntBound.unbounded()"
        if self.lower == 0 and self.upper == MAXINT and self._are_knownbits_implied():
            return "IntBound.nonnegative()"
        if self.is_constant():
            return "IntBound.from_constant(%s)" % self._to_dec_or_hex_str_heuristics(self.get_constant_int())
        s_bounds = "%s, %s" % (self._to_dec_or_hex_str_heuristics(self.lower),
                               self._to_dec_or_hex_str_heuristics(self.upper))

        if self._are_knownbits_implied():
            return "IntBound(%s)" % s_bounds

        s_tnum = "r_uint(%s), r_uint(%s)" % (bin(intmask(self.tvalue)), bin(intmask(self.tmask)))
        if self._are_bounds_implied():
            return "IntBound.from_knownbits(%s)" % s_tnum
        return "IntBound(%s, %s)" % (s_bounds, s_tnum)

    def __str__(self):
        if self.is_constant():
            return '(%s)' % self._to_dec_or_hex_str_heuristics(self.get_constant_int())
        if self.lower == 0 and self.upper == 1:
            return '(bool)'
        if self.lower == MININT:
            lower = ''
        else:
            lower = '%s <= ' % self._to_dec_or_hex_str_heuristics(self.lower)
        if self.upper == MAXINT:
            upper = ''
        else:
            upper = ' <= %s' % self._to_dec_or_hex_str_heuristics(self.upper)
        s = self.knownbits_string()
        if "0" not in s and "1" not in s:
            s = '?'
        else:
            # replace the longest sequence of same characters by ...
            prev_char = s[0]
            count = 0
            max_length = 0
            max_char = '\x00'
            start_pos = 0
            max_pos = -1
            for pos, char in enumerate(s):
                if char == prev_char:
                    count += 1
                else:
                    if count > max_length:
                        max_length = count
                        max_char = prev_char
                        max_pos = start_pos
                    prev_char = char
                    count = 1
                    start_pos = pos
            if count > max_length:
                max_length = count
                max_char = prev_char
                max_pos = start_pos
            if max_length > 5:
                assert max_pos >= 0
                s = s[:max_pos] + max_char + "..." + max_char + s[max_pos + max_length:]
            s = '0b' + s
        return '(%s%s%s)' % (lower, s, upper)

    def set_tvalue_tmask(self, tvalue, tmask):
        changed = self.tvalue != tvalue or self.tmask != tmask
        if changed:
            self.tvalue = tvalue
            self.tmask = tmask
            self.shrink()
        return changed

    def make_le(self, other):
        """
        Sets the bounds of `self` so that it only contains values lower than or
        equal to the values contained in `other`. Returns `True` iff the bound
        was updated. Will raise InvalidLoop if the resulting interval is empty.
        Mutates `self`.
        """
        return self.make_le_const(other.upper)

    def make_le_const(self, value):
        """
        Sets the bounds of `self` so that it only contains values lower than or
        equal to `value`. Returns `True` iff the bound was updated. Will raise
        InvalidLoop if the resulting interval is empty.
        Mutates `self`.
        """
        if value < self.upper:
            if value < self.lower:
                raise InvalidLoop
            self.upper = value
            self.shrink()
            return True
        return False

    def make_lt(self, other):
        """
        Sets the bounds of `self` so that it only contains values lower than
        the values contained in `other`. Returns `True` iff the bound was
        updated. Will raise InvalidLoop if the resulting interval is empty.
        (Mutates `self`.)
        """
        return self.make_lt_const(other.upper)

    def make_lt_const(self, value):
        """
        Sets the bounds of `self` so that it only contains values lower than
        `value`. Returns `True` iff the bound was updated. Will raise
        InvalidLoop if the resulting interval is empty. (Mutates `self`.)
        """
        if value == MININT:
            raise InvalidLoop("intbound can't be made smaller than MININT")
        return self.make_le_const(value - 1)

    def make_ge(self, other):
        """
        Sets the bounds of `self` so that it only contains values greater than
        or equal to the values contained in `other`. Returns `True` iff the
        bound was updated. Will raise InvalidLoop if the resulting interval is
        empty. (Mutates `self`.)
        """
        return self.make_ge_const(other.lower)

    def make_ge_const(self, value):
        """
        Sets the bounds of `self` so that it only contains values greater than
        or equal to `value`. Returns `True` iff the bound was updated. Will
        raise InvalidLoop if the resulting interval is empty. (Mutates `self`.)
        """
        if value > self.lower:
            if value > self.upper:
                raise InvalidLoop
            self.lower = value
            self.shrink()
            return True
        return False

    def make_gt(self, other):
        """
        Sets the bounds of `self` so that it only contains values greater than
        the values contained in `other`. Returns `True` iff the bound was
        updated. Will raise InvalidLoop if the resulting interval is empty.
        (Mutates `self`.)
        """
        return self.make_gt_const(other.lower)

    def make_gt_const(self, value):
        """
        Sets the bounds of `self` so that it only contains values greater than
        `value`. Returns `True` iff the bound was updated. Will raise
        InvalidLoop if the resulting interval is empty. (Mutates `self`.)
        """
        if value == MAXINT:
            raise InvalidLoop
        return self.make_ge_const(value + 1)

    def make_eq_const(self, intval):
        """
        Sets the properties of this abstract integer
        so that it is constant and equals `intval`.
        (Mutates `self`.)
        """
        if not self.contains(intval):
            raise InvalidLoop("constant int is outside of interval")
        self.upper = intval
        self.lower = intval
        self.tvalue = r_uint(intval)
        self.tmask = r_uint(0)

    def make_ne_const(self, intval):
        if self.lower < intval == self.upper:
            self.upper -= 1
            self.shrink()
            return True
        if self.lower == intval < self.upper:
            self.lower += 1
            self.shrink()
            return True
        return False

    def is_constant(self):
        """
        Returns `True` iff this abstract integer
        does contain only one concrete integer.
        """
        # both the bounds and the tnum encode the concrete integer
        res = self.lower == self.upper
        assert res == (self.tmask == r_uint(0))
        if res:
            assert self.lower == intmask(self.tvalue)
        return res

    def get_constant_int(self):
        """
        Returns the only integer contained in this abstract integer. Caller
        needs to check that `.is_constant()` returns True, before calling.
        """
        assert self.is_constant()
        return self.lower

    def known_eq_const(self, value):
        """
        Returns `True` iff this abstract integer contains only one (1) integer
        that does equal `value`.
        """
        if not self.is_constant():
            return False
        else:
            return self.lower == value

    def known_lt_const(self, value):
        """
        Returns `True` iff each number contained in this abstract integer is
        lower than `value`.
        """
        return self.upper < value

    def known_le_const(self, value):
        """
        Returns `True` iff each number contained in this abstract integer is
        lower than or equal to `value`.
        """
        return self.upper <= value

    def known_gt_const(self, value):
        """
        Returns `True` iff each number contained in this abstract integer is
        greater than `value`.
        """
        return self.lower > value

    def known_ge_const(self, value):
        """
        Returns `True` iff each number contained in this abstract integer is
        greater than equal to `value`.
        """
        return self.lower >= value

    def known_lt(self, other):
        """
        Returns `True` iff each number contained in this abstract integer is
        lower than each integer contained in `other`.
        """
        return self.known_lt_const(other.lower)

    def known_le(self, other):
        """
        Returns `True` iff each number contained in this abstract integer is
        lower than or equal to each integer contained in `other`.
        """
        return self.known_le_const(other.lower)

    def known_gt(self, other):
        """
        Returns `True` iff each number contained in this abstract integer is
        greater than each integer contained in `other`.
        """
        return other.known_lt(self)

    def known_ge(self, other):
        """
        Returns `True` iff each number contained in this abstract integer is
        greater than or equal to each integer contained in `other`.
        """
        return other.known_le(self)

    def known_ne(self, other):
        """ return True if the sets of numbers self and other must be disjoint.
        """
        # easy cases part 1: ranges don't overlap
        if self.known_lt(other):
            return True
        if self.known_gt(other):
            return True
        # easy case part 2: check whether the knownbits contradict
        both_known = self.tmask | other.tmask
        if unmask_zero(self.tvalue, both_known) != unmask_zero(other.tvalue, both_known):
            return True
        # for more complicated interactions between ranges and knownbits use
        # the logic in intersect
        newself = self.clone()
        try:
            newself.intersect(other)
        except InvalidLoop:
            return True
        return False

    def known_nonnegative(self):
        """
        Returns `True` if this abstract integer only contains numbers greater
        than or equal to `0` (zero).
        """
        return 0 <= self.lower

    def make_unsigned_le(self, other):
        if other.known_nonnegative():
            return self.intersect_const(0, other.upper)
        return False

    def make_unsigned_lt(self, other):
        if other.known_nonnegative():
            assert other.upper >= 0
            if other.upper == 0:
                raise InvalidLoop
            return self.intersect_const(0, other.upper - 1)
        return False

    def make_unsigned_ge(self, other):
        if other.upper < 0:
            changed = self.make_lt_const(0)
            return self.make_ge(other) or changed
        if self.known_nonnegative() and other.known_nonnegative():
            return self.make_ge(other)
        return False

    def make_unsigned_gt(self, other):
        if other.upper < 0:
            changed = self.make_lt_const(0)
            return self.make_gt(other) or changed
        if self.known_nonnegative() and other.known_nonnegative():
            return self.make_gt(other)
        return False

    def _known_same_sign(self, other):
        # return True if self and other are both either known non-negative or
        # both known negative
        if self.known_nonnegative() and other.known_nonnegative():
            return True
        return self.known_lt_const(0) and other.known_lt_const(0)

    def known_unsigned_lt(self, other):
        """
        Returns `True` iff each unsigned integer contained in this abstract
        integer is lower than each unsigned integer contained in `other`.
        """
        # if they have the same sign, we can reason with signed comparison
        # see test_uint_cmp_equivalent_int_cmp_if_same_sign
        if self._known_same_sign(other) and self.known_lt(other):
            return True
        other_min_unsigned_by_knownbits = other.get_minimum_unsigned_by_knownbits()
        self_max_unsigned_by_knownbits = self.get_maximum_unsigned_by_knownbits()
        return self_max_unsigned_by_knownbits < other_min_unsigned_by_knownbits

    def known_unsigned_le(self, other):
        """
        Returns `True` iff each unsigned integer contained in this abstract
        integer is lower or equal than each unsigned integer contained in
        `other`. """
        # if they have the same sign, we can reason with signed comparison
        if self._known_same_sign(other) and self.known_le(other):
            return True
        other_min_unsigned_by_knownbits = other.get_minimum_unsigned_by_knownbits()
        self_max_unsigned_by_knownbits = self.get_maximum_unsigned_by_knownbits()
        return self_max_unsigned_by_knownbits <= other_min_unsigned_by_knownbits

    def known_unsigned_gt(self, other):
        """
        Returns `True` iff each unsigned integer contained in this abstract
        integer is greater than each unsigned integer contained in `other`.
        """
        return other.known_unsigned_lt(self)

    def known_unsigned_ge(self, other):
        """
        Returns `True` iff each unsigned integer contained in this abstract
        integer is greater or equal than each unsigned integer contained in
        `other`.
        """
        return other.known_unsigned_le(self)

    def get_minimum_unsigned_by_knownbits(self):
        """
        Returns the minimum unsigned number, but only using the knownbits as
        information."""
        return unmask_zero(self.tvalue, self.tmask)

    def get_maximum_unsigned_by_knownbits(self):
        """
        returns the maximum unsigned number, but only using the knownbits as
        information."""
        return unmask_one(self.tvalue, self.tmask)

    def _get_minimum_signed_by_knownbits(self):
        """ for internal use only!
        returns the minimum signed number, but only using the knownbits as
        information."""
        return self.intmask(self.tvalue | msbonly(self.tmask))

    def _get_maximum_signed_by_knownbits(self):
        """ for internal use only!
        returns the maximum signed number, but only using the knownbits as
        information."""
        unsigned_mask = self.tmask & ~msbonly(self.tmask)
        return self.intmask(self.tvalue | unsigned_mask)

    def _get_minimum_signed_by_knownbits_atleast(self, threshold=MININT):
        """ for internal use only!
        return the smallest number permitted by the known bits that is above
        (or equal) threshold. will raise InvalidLoop if no such number exists.
        """
        if self._get_maximum_signed_by_knownbits() < threshold:
            raise InvalidLoop("threshold and knownbits don't overlap")
        min_by_knownbits = self._get_minimum_signed_by_knownbits()
        if min_by_knownbits > self.upper:
            raise InvalidLoop("range and knownbits don't overlap")
        if min_by_knownbits >= threshold:
            return min_by_knownbits
        # see "Sharpening Constraint Programming
        #      approaches for Bit-Vector Theory"
        u_min_threshold = r_uint(threshold)
        # create our working value, the to-be minimum
        working_min, cl2set, set2cl = self._helper_min_max_prepare(u_min_threshold)
        if working_min == u_min_threshold:
            return threshold
        elif cl2set > set2cl:
            return self._helper_min_case1(working_min, cl2set)
        else:
            return self._helper_min_case2(working_min, set2cl)

    @always_inline
    def _helper_min_max_prepare(self, u_threshold):
        working_value = u_threshold # start at given threshold
        working_value &= unmask_one(self.tvalue, self.tmask) # clear known 0s
        working_value |= self.tvalue # set known 1s
        # inspect changed bits
        cl2set = ~u_threshold & working_value
        set2cl = u_threshold & ~working_value
        return working_value, cl2set, set2cl

    @always_inline
    def _helper_min_case1(self, working_min, cl2set):
        # we have set the correct bit already
        clear_mask = leading_zeros_mask(self._urshift(cl2set, 1))
        working_min &= clear_mask | ~self.tmask
        return self.intmask(working_min)

    @always_inline
    def _helper_min_case2(self, working_min, set2cl):
        # flip the sign bit to handle -1 -> 0 overflow
        working_min = flip_msb(working_min)
        # we have to find the proper bit to set...
        possible_bits = ~working_min \
                        & self.tmask \
                        & leading_zeros_mask(set2cl)
        bit_to_set = lowest_set_bit_only(possible_bits)
        working_min |= bit_to_set
        # and clear all lower than that
        clear_mask = leading_zeros_mask(bit_to_set) \
                     | bit_to_set | ~self.tmask
        working_min &= clear_mask
        return self.intmask(flip_msb(working_min))

    def _get_maximum_signed_by_knownbits_atmost(self, threshold=MAXINT):
        """ for internal use only!
        return the largest number permitted by the known bits that is below or
        equal to threshold. will raise InvalidLoop if no such number exists.
        """
        if self._get_minimum_signed_by_knownbits() > threshold:
            raise InvalidLoop("threshold and knownbits don't overlap")
        max_by_knownbits = self._get_maximum_signed_by_knownbits()
        if max_by_knownbits < self.lower:
            raise InvalidLoop("range and knownbits don't overlap")
        if max_by_knownbits <= threshold:
            return max_by_knownbits
        # see "Sharpening Constraint Programming
        #      approaches for Bit-Vector Theory"
        u_max_threshold = r_uint(threshold)
        # now create our working value, the to-be maximum
        working_max, cl2set, set2cl = self._helper_min_max_prepare(u_max_threshold)
        if working_max == u_max_threshold:
            return threshold
        elif cl2set < set2cl:
            # we have cleared the right bit already
            result = self._helper_max_case1(working_max, set2cl)
        else:
            result = self._helper_max_case2(working_max, cl2set)
        assert result <= threshold
        return result

    def _helper_max_case1(self, working_max, set2cl):
        # we have cleared the right bit already
        set_mask = next_pow2_m1(self._urshift(set2cl, 1)) & self.tmask
        working_max |= set_mask
        return self.intmask(working_max)

    def _helper_max_case2(self, working_max, cl2set):
        # flip the sign bit to handle 1 -> 0 overflow
        working_max = flip_msb(working_max)
        # find the right bit to clear
        possible_bits = working_max \
                        & self.tmask \
                        & leading_zeros_mask(cl2set)
        bit_to_clear = lowest_set_bit_only(possible_bits)
        working_max &= ~bit_to_clear
        # and set all lower than that
        set_mask = next_pow2_m1(self._urshift(bit_to_clear, 1)) & self.tmask
        working_max |= set_mask
        return self.intmask(flip_msb(working_max))


    def _get_minimum_signed(self):
        """ for tests only """
        ret_b = self.lower
        result = self._get_minimum_signed_by_knownbits_atleast(ret_b)
        assert isinstance(result, int)
        return result

    def _get_maximum_signed(self):
        """ for tests only """
        ret_b = self.upper
        result = self._get_maximum_signed_by_knownbits_atmost(ret_b)
        assert isinstance(result, int)
        return result

    def intersect(self, other):
        """
        Mutates `self` so that it contains integers that are contained in
        `self` and `other`, and only those. Basically intersection of sets.
        Throws InvalidLoop if `self` and `other` "disagree", meaning the result
        would not contain any integers.
        """
        if self.known_gt(other) or self.known_lt(other):
            # they don't overlap, which makes the loop invalid
            # this never happens in regular linear traces, but it can happen in
            # combination with unrolling/loop peeling
            raise InvalidLoop("two integer ranges don't overlap")

        r = self.intersect_const(other.lower, other.upper, do_shrinking=False)

        tvalue, tmask, valid = self._tnum_intersect(other.tvalue, other.tmask)
        if not valid:
            raise InvalidLoop("knownbits contradict each other")
        # calculate intersect value and mask
        if self.tmask != tmask:
            # this can also raise InvalidLoop, if the ranges and knownbits
            # contradict in more complicated ways
            r = self.set_tvalue_tmask(tvalue, tmask) # this shrinks
            assert r
        elif r:
            # we didn't shrink yet
            self.shrink()
            assert self._debug_check()
        return r

    @always_inline
    def _tnum_intersect(self, other_tvalue, other_tmask):
        union_val = self.tvalue | other_tvalue
        either_known = self.tmask & other_tmask
        both_known = self.tmask | other_tmask
        unmasked_self = unmask_zero(self.tvalue, both_known)
        unmasked_other = unmask_zero(other_tvalue, both_known)
        tvalue = unmask_zero(union_val, either_known)
        valid = unmasked_self == unmasked_other
        return tvalue, either_known, valid

    def intersect_const(self, lower, upper, do_shrinking=True):
        """
        Mutates `self` so that it contains integers that are contained in
        `self` and the range [`lower`, `upper`], and only those. Basically
        intersection of sets.
        """
        changed = False
        if lower > self.lower:
            if lower > self.upper:
                raise InvalidLoop
            self.lower = lower
            changed = True
        if upper < self.upper:
            if upper < self.lower:
                raise InvalidLoop
            self.upper = upper
            changed = True
        if changed and do_shrinking:
            self.shrink()
        return changed

    def add(self, value):
        return self.add_bound(self.from_constant(value))

    def add_bound(self, other):
        """
        Adds the `other` abstract integer to `self` and returns the result.
        Must be correct in the presence of possible overflows.
        (Does not mutate `self`.)
        """

        tvalue, tmask = self._tnum_add(other)

        # the lower and upper logic is proven in test_prove_add_bounds_logic
        try:
            lower = ovfcheck(self.lower + other.lower)
        except OverflowError:
            return IntBound.from_knownbits(tvalue, tmask)
        try:
            upper = ovfcheck(self.upper + other.upper)
        except OverflowError:
            return IntBound.from_knownbits(tvalue, tmask)
        return IntBound(lower, upper, tvalue, tmask)

    @always_inline
    def _tnum_add(self, other):
        sum_values = self.tvalue + other.tvalue
        sum_masks = self.tmask + other.tmask
        all_carries = sum_values + sum_masks
        val_carries = all_carries ^ sum_values
        tmask = self.tmask | other.tmask | val_carries
        tvalue = unmask_zero(sum_values, tmask)
        return tvalue, tmask

    def add_bound_cannot_overflow(self, other):
        """ returns True if self + other can never overflow """
        try:
            ovfcheck(self.upper + other.upper)
            ovfcheck(self.lower + other.lower)
        except OverflowError:
            return False
        return True

    def add_bound_no_overflow(self, other):
        """ return the bound that self + other must have, if no overflow occured,
        eg after an int_add_ovf(...), guard_no_overflow() """
        tvalue, tmask = self._tnum_add(other)

        # returning add_bound is always correct, but let's improve the range
        lower = self._add_check_overflow(self.lower, other.lower, MININT)
        upper = self._add_check_overflow(self.upper, other.upper, MAXINT)
        return self.new(lower, upper, tvalue, tmask)

    def mul2_bound_no_overflow(self):
        """ return the bound that self + self must have, if no overflow occured,
        eg after an int_add_ovf(a, a), guard_no_overflow() """
        tvalue, tmask = self._tnum_lshift(1) # to know that the result is even

        # returning add_bound is always correct, but let's improve the range
        lower = self._add_check_overflow(self.lower, self.lower, MININT)
        upper = self._add_check_overflow(self.upper, self.upper, MAXINT)
        return self.new(lower, upper, tvalue, tmask)

    def sub_bound(self, other):
        """
        Subtracts the `other` abstract integer from `self` and returns the
        result. (Does not mutate `self`.)
        """
        tvalue, tmask = self._tnum_sub(other)
        # the lower and upper logic is proven in test_prove_sub_bound_logic
        try:
            lower = ovfcheck(self.lower - other.upper)
        except OverflowError:
            return IntBound.from_knownbits(tvalue, tmask)
        try:
            upper = ovfcheck(self.upper - other.lower)
        except OverflowError:
            return IntBound.from_knownbits(tvalue, tmask)
        return IntBound(lower, upper, tvalue, tmask)

    def _tnum_sub(self, other):
        diff_values = self.tvalue - other.tvalue
        val_borrows = (diff_values + self.tmask) ^ (diff_values - other.tmask)
        tmask = self.tmask | other.tmask | val_borrows
        tvalue = unmask_zero(diff_values, tmask)
        return tvalue, tmask

    def sub_bound_cannot_overflow(self, other):
        try:
            ovfcheck(self.lower - other.upper)
            ovfcheck(self.upper - other.lower)
        except OverflowError:
            return False
        return True

    def sub_bound_no_overflow(self, other):
        """ return the bound that self - other must have, if no overflow occured,
        eg after an int_sub_ovf(...), guard_no_overflow() """
        tvalue, tmask = self._tnum_sub(other)
        # returning sub_bound is always correct, but let's improve the range
        lower = self._sub_check_overflow(self.lower, other.upper, MININT)
        upper = self._sub_check_overflow(self.upper, other.lower, MAXINT)
        return self.new(lower, upper, tvalue, tmask)

    def mul_bound(self, other):
        """
        Multiplies the `other` abstract
        integer with `self` and returns the
        result.
        (Does not mutate `self`.)
        """
        try:
            vals = (ovfcheck(self.upper * other.upper),
                    ovfcheck(self.upper * other.lower),
                    ovfcheck(self.lower * other.upper),
                    ovfcheck(self.lower * other.lower))
            return IntBound(min4(vals), max4(vals))
        except OverflowError:
            return IntBound.unbounded()
    mul_bound_no_overflow = mul_bound # can be improved

    def mul_bound_cannot_overflow(self, other):
        try:
            ovfcheck(self.upper * other.upper)
            ovfcheck(self.upper * other.lower)
            ovfcheck(self.lower * other.upper)
            ovfcheck(self.lower * other.lower)
        except OverflowError:
            return False
        return True

    def square_bound_no_overflow(self):
        """
        returns the bound of self**2, if no overflow occurred.
        """
        val0 = saturating_mul(self.lower, self.lower) # self.lower**2, or MAXINT
        val1 = saturating_mul(self.upper, self.upper) # self.upper**2, or MAXINT
        lower = min(val0, val1)
        upper = max(val0, val1)
        if not same_sign(self.upper, self.lower): # 0 is contained in the range
            lower = 0
        return IntBound(lower, upper)

    def py_div_bound(self, other):
        """
        Divides this abstract integer by the
        `other` abstract integer and returns
        the result.
        (Does not mutate `self`.)
        """
        # we need to make sure that the 0 is not in the interval because
        # otherwise [-4, 4] / [-4, 4] would return [-1, 1], which is nonsense
        # see test_knownbits_div_bug. the first part of the check is not
        # enough, because 0 could be excluded by the known bits
        if not other.contains(0) and not (other.lower < 0 < other.upper):
            try:
                # this gives the bounds for 'int_py_div', so use the
                # Python-style handling of negative numbers and not
                # the C-style one
                vals = (ovfcheck(self.upper / other.upper),
                        ovfcheck(self.upper / other.lower),
                        ovfcheck(self.lower / other.upper),
                        ovfcheck(self.lower / other.lower))
                return IntBound(min4(vals), max4(vals))
            except OverflowError:
                pass
        return IntBound.unbounded()

    def mod_bound(self, other):
        """
        Calculates the mod of this abstract
        integer by the `other` abstract
        integer and returns the result.
        (Does not mutate `self`.)
        """
        r = IntBound.unbounded()
        if other.is_constant() and other.get_constant_int() == 0:
            return IntBound.unbounded()
        # with Python's modulo:  0 <= (x % pos) < pos
        #                        neg < (x % neg) <= 0
        # see test_prove_mod_bound_logic
        if other.upper > 0:
            upper = other.upper - 1
        else:
            upper = 0
        if other.lower < 0:
            lower = other.lower + 1
        else:
            lower = 0
        return IntBound(lower, upper)

    def lshift_bound(self, other):
        """
        Shifts this abstract integer `other`
        bits to the left, where `other` is
        another abstract integer.
        (Does not mutate `self`.)
        """

        tvalue, tmask = TNUM_UNKNOWN
        if other.is_constant():
            c_other = other.get_constant_int()
            if c_other >= LONG_BIT:
                 tvalue, tmask = TNUM_KNOWN_ZERO
            elif 0 <= c_other < LONG_BIT:
                tvalue, tmask = self._tnum_lshift(c_other)
            # else: bits are unknown because arguments invalid

        if other.known_nonnegative() and other.known_lt_const(LONG_BIT):
            try:
                vals = (ovfcheck(self.upper << other.upper),
                        ovfcheck(self.upper << other.lower),
                        ovfcheck(self.lower << other.upper),
                        ovfcheck(self.lower << other.lower))
                return IntBound(min4(vals), max4(vals), tvalue, tmask)
            except (OverflowError, ValueError):
                pass

        return IntBound.from_knownbits(tvalue, tmask)

    @always_inline
    def _tnum_lshift(self, c_other):
        # use signed integer sign extension logic
        tvalue = self.tvalue << c_other
        tmask = self.tmask << c_other
        return tvalue, tmask

    def rshift_bound(self, other):
        """
        Shifts this abstract integer `other` bits to the right, where `other`
        is another abstract integer, and extends its sign. This is the
        arithmetic shift on signed integers, ie the shifted in values are 0/1,
        depending on the sign.
        (Does not mutate `self`.)
        """

        tvalue, tmask = TNUM_UNKNOWN
        if other.is_constant():
            c_other = other.get_constant_int()
            if c_other >= LONG_BIT:
                # shift value out to the right, but do sign extend
                if msbonly(self.tmask): # sign-extend mask
                    tvalue, tmask = TNUM_UNKNOWN
                elif msbonly(self.tvalue): # sign-extend value
                    tvalue, tmask = TNUM_KNOWN_BITWISEONE
                else: # sign is 0 on both
                    tvalue, tmask = TNUM_KNOWN_ZERO
            elif c_other >= 0:
                tvalue, tmask = self._tnum_rshift(c_other)
            # else: bits are unknown because arguments invalid

        lower = MININT
        upper = MAXINT
        if other.known_nonnegative() and other.known_lt_const(LONG_BIT):
            vals = (self.upper >> other.upper,
                    self.upper >> other.lower,
                    self.lower >> other.upper,
                    self.lower >> other.lower)
            lower = min4(vals)
            upper = max4(vals)
        return IntBound(lower, upper, tvalue, tmask)

    @always_inline
    def _tnum_rshift(self, c_other):
        # use signed integer sign extension logic
        tvalue = self.r_uint(self.intmask(self.tvalue) >> c_other)
        tmask = self.r_uint(self.intmask(self.tmask) >> c_other)
        return tvalue, tmask

    def lshift_bound_cannot_overflow(self, other):
        """ returns True if self << other can never overflow """
        if other.known_nonnegative() and \
           other.known_lt_const(LONG_BIT):
            try:
                ovfcheck(self.upper << other.upper)
                ovfcheck(self.upper << other.lower)
                ovfcheck(self.lower << other.upper)
                ovfcheck(self.lower << other.lower)
                return True
            except (OverflowError, ValueError):
                pass
        return False

    def urshift_bound(self, other):
        """
        Shifts this abstract integer `other` bits to the right, where `other`
        is another abstract integer, *without* extending its sign. (Does not
        mutate `self`.)
        """

        # this seems to always be the signed variant..?
        lower = MININT
        upper = MAXINT
        tvalue, tmask = TNUM_UNKNOWN
        if other.is_constant():
            c_other = other.get_constant_int()
            if c_other >= LONG_BIT:
                # no sign to extend, we get constant 0
                tvalue, tmask = TNUM_KNOWN_ZERO
            elif c_other >= 0:
                tvalue, tmask = self._tnum_urshift(c_other)
                if self.lower >= 0:
                    upper = intmask(r_uint(self.upper) >> c_other)
                    lower = intmask(r_uint(self.lower) >> c_other)
            # else: bits are unknown because arguments invalid
        return IntBound(lower, upper, tvalue, tmask)

        # we don't do bounds on unsigned
        return IntBound.from_knownbits(tvalue, tmask)

    @always_inline
    def _tnum_urshift(self, c_other):
        tvalue = self._urshift(self.tvalue, c_other)
        tmask = self._urshift(self.tmask, c_other)
        return tvalue, tmask

    def and_bound(self, other):
        """
        Performs bit-wise AND of this abstract integer and the `other`,
        returning its result. (Does not mutate `self`.)
        """

        pos1 = self.known_nonnegative()
        pos2 = other.known_nonnegative()
        # the next three if-conditions are proven by test_prove_and_bound_logic
        lower = MININT
        upper = MAXINT
        if pos1 or pos2:
            lower = 0
        if pos1:
            upper = self.upper
        if pos2:
            upper = min(upper, other.upper)

        res_tvalue, res_tmask = self._tnum_and(other)
        return IntBound(lower, upper, res_tvalue, res_tmask)

    @always_inline
    def _tnum_and(self, other):
        self_pmask = self.tvalue | self.tmask
        other_pmask = other.tvalue | other.tmask
        and_vals = self.tvalue & other.tvalue
        return and_vals, self_pmask & other_pmask & ~and_vals

    def or_bound(self, other):
        """
        Performs bit-wise OR of this abstract integer and the `other`,
        returning its result. (Does not mutate `self`.)
        """

        tvalue, tmask = self._tnum_or(other)
        return self.from_knownbits(tvalue, tmask)

    @always_inline
    def _tnum_or(self, other):
        union_vals = self.tvalue | other.tvalue
        union_masks = self.tmask | other.tmask
        return union_vals, union_masks & ~union_vals

    def xor_bound(self, other):
        """
        Performs bit-wise XOR of this abstract integer and the `other`,
        returning its result.
        (Does not mutate `self`.)
        """
        tvalue, tmask = self._tnum_xor(other)
        return self.from_knownbits(tvalue, tmask)

    @always_inline
    def _tnum_xor(self, other):
        xor_vals = self.tvalue ^ other.tvalue
        union_masks = self.tmask | other.tmask
        return unmask_zero(xor_vals, union_masks), union_masks

    def neg_bound(self):
        """
        Arithmetically negates this abstract integer and returns the result.
        (Does not mutate `self`.)
        """
        res = self.invert_bound()
        res = res.add(1)
        return res

    def invert_bound(self):
        """
        Performs bit-wise NOT on this abstract integer returning its result.
        (Does not mutate `self`.)
        """
        upper = ~self.lower
        lower = ~self.upper
        tvalue = unmask_zero(~self.tvalue, self.tmask)
        tmask = self.tmask
        return self.new(lower, upper, tvalue, tmask)

    def contains(self, value):
        """
        Returns `True` iff this abstract integer contains the given `value`.
        """

        assert not isinstance(value, IntBound)

        if not we_are_translated():
            assert not isinstance(value, long)
        if not isinstance(value, int):
            if (self.lower == MININT and self.upper == MAXINT):
                return True # workaround for address as int
        if value < self.lower:
            return False
        if value > self.upper:
            return False

        u_vself = unmask_zero(self.tvalue, self.tmask)
        u_value = unmask_zero(r_uint(value), self.tmask)
        if u_vself != u_value:
            return False

        return True

    def is_within_range(self, lower, upper):
        """
        Check if all the numbers contained in this instance have are between
        lower and upper.
        """
        return lower <= self.lower and self.upper <= upper

    def clone(self):
        """
        Returns a copy of this abstract integer.
        """
        res = IntBound(self.lower, self.upper,
                       self.tvalue, self.tmask)
        return res

    def make_guards(self, box, guards, optimizer):
        """
        Generates guards from the information we have about the numbers this
        abstract integer contains.
        """
        if self.is_constant():
            guards.append(ResOperation(rop.GUARD_VALUE,
                                       [box, ConstInt(self.upper)]))
            return
        if self.lower > MININT:
            bound = self.lower
            op = ResOperation(rop.INT_GE, [box, ConstInt(bound)])
            guards.append(op)
            op = ResOperation(rop.GUARD_TRUE, [op])
            guards.append(op)
        if self.upper < MAXINT:
            bound = self.upper
            op = ResOperation(rop.INT_LE, [box, ConstInt(bound)])
            guards.append(op)
            op = ResOperation(rop.GUARD_TRUE, [op])
            guards.append(op)
        if not self._are_knownbits_implied():
            op = ResOperation(rop.INT_AND, [box, ConstInt(intmask(~self.tmask))])
            guards.append(op)
            op = ResOperation(rop.GUARD_VALUE, [op, ConstInt(intmask(self.tvalue))])
            guards.append(op)

    def is_bool(self):
        """
        Returns `True` iff self is exactly the set {0, 1}
        """
        return (self.known_nonnegative() and self.known_le_const(1))

    def is_unbounded(self):
        return (self.lower == MININT and self.upper == MAXINT and
                self.tvalue == r_uint(0) and self.tmask == r_uint(-1))


    def make_bool(self):
        """
        Mutates this abstract integer so that it does represent a conventional
        boolean value.
        (Mutates `self`.)
        """
        self.intersect_const(0, 1)

    def getconst(self):
        """
        Returns ConstInt with the only integer contained in this abstract
        integer. Caller needs to check that `.is_constant()` returns True,
        before calling.
        """
        return ConstInt(self.get_constant_int())

    def getnullness(self):
        """
        Returns information about whether this this abstract integer is known
        to be zero or not to be zero.
        """
        if self.known_gt_const(0) or \
           self.known_lt_const(0) or \
           self.tvalue != 0:
            return INFO_NONNULL
        if self.is_constant() and self.get_constant_int() == 0:
            return INFO_NULL
        return INFO_UNKNOWN

    def widen(self):
        info = self.clone()
        info.widen_update()
        return info

    def widen_update(self):
        if self.lower < MININT / 2:
            self.lower = MININT
        if self.upper > MAXINT / 2:
            self.upper = MAXINT
        self.tvalue, self.tmask = TNUM_UNKNOWN
        self.shrink()


    def and_bound_backwards(self, result):
        """
        result == int_and(self, other)
        We want to learn some bits about other, using the information from self
        and result_int.

        regular &:
                  other
         &  0   1   ?
         0  0   0   0
         1  0   1   ?
         ?  0   ?   ?   <- result
        self

        backwards & (this one):
                  self
            0   1   ?
         0  ?   0   ?
         1  X   1   1
         ?  ?   ?   ?   <- other
        result

        (X marks an inconsistent result).

        We can see that we only learn something about other at the places
        where a bit from self is 1. At those places, the corresponding bit in
        other has to be the corresponding bit in result.
        """

        tvalue, tmask, valid = self._tnum_and_backwards(result)
        if not valid:
            raise InvalidLoop("inconsistency in and_bound_backwards")
        return IntBound.from_knownbits(tvalue, tmask)

    @always_inline
    def _tnum_and_backwards(self, result):
        # in all the places where the result is 1 both arguments have to 1. in
        # the places where result is 0, other has to be 0 iff self is known 1.
        tvalue = result.tvalue
        tmask = ((~self.tvalue) | result.tmask) & ~tvalue
        # if we have a place where result is 1 but self is 0, then we are
        # inconsistent
        inconsistent = result.tvalue & ~self.tmask & ~self.tvalue
        return tvalue, tmask, inconsistent == 0

    def or_bound_backwards(self, result):
        """
        result_int == int_or(self, other)
        We want to refine our knowledge about other
        using this information

        regular |:
                  other
         &  0   1   ?
         0  0   1   ?
         1  1   1   1
         ?  ?   1   ?   <- result
        self

        backwards | (this one):
                  self
            0   1   ?
         0  0   X   0
         1  1   ?   ?
         ?  ?   ?   ?   <- other (where X=invalid)
        result

        """
        tvalue, tmask, valid = self._tnum_or_backwards(result)
        if not valid:
            raise InvalidLoop("inconsistency in or_bound_backwards")
        return IntBound.from_knownbits(tvalue, tmask)

    @always_inline
    def _tnum_or_backwards(self, result):
        # in all the places where the result is 0 both arguments have to be 0.
        zeros = (~result.tmask & ~result.tvalue)
        # apart from that, in the places where self is 0 and where result is 1
        # other must be 1
        tvalue = (result.tvalue & ~self.tvalue & ~self.tmask)
        tmask = ~(zeros | tvalue)
        # if we have a place where result is 0 but self is 1, then we are
        # inconsistent
        inconsistent = self.tvalue & zeros
        return tvalue, tmask, inconsistent == 0

    def rshift_bound_backwards(self, other):
        """
        Performs a `urshift`/`rshift` backwards on `self`. Basically
        left-shifts `self` by `other` binary digits, filling the lower part
        with ?, and returns the result.
        """
        if not other.is_constant():
            return IntBound.unbounded()
        c_other = other.get_constant_int()
        tvalue, tmask = TNUM_UNKNOWN
        if 0 <= c_other < LONG_BIT:
            tvalue = self.tvalue << r_uint(c_other)
            tmask = self.tmask << r_uint(c_other)
            # shift ? in from the right,
            tmask |= (r_uint(1) << r_uint(c_other)) - 1
        return IntBound.from_knownbits(tvalue, tmask)
    urshift_bound_backwards = rshift_bound_backwards

    def lshift_bound_backwards(self, other):
        """
        Performs a `lshift` backwards on `self`. Basically right-shifts `self`
        by `other` binary digits, filling the upper part with ?, and returns
        the result.
        """
        if not other.is_constant():
            return IntBound.unbounded()
        c_other = r_uint(other.get_constant_int())
        tvalue, tmask = TNUM_UNKNOWN
        if 0 <= c_other < LONG_BIT:
            tvalue = self.tvalue >> c_other
            tmask = self.tmask >> c_other
            # shift ? in from the left,
            s_tmask = ~(r_uint(-1) >> c_other)
            tmask |= s_tmask
            inconsistent = self.tvalue & ((1 << c_other) - 1)
            if inconsistent:
                raise InvalidLoop("lshift_bound_backwards inconsistent known bits")
        return IntBound.from_knownbits(tvalue, tmask)

    def shrink(self):
        """ Shrink the bounds and the knownbits to be more precise, but without
        changing the set of integers that is represented by self.

        Here's a diagram to show what is happening. This is the number line:
        MININT <---------0-----------------------------------------------> MAXINT

        We have a range in the number line
                           [lower     ...       upper]

        We also known bits, they represent a set of ints maybe looking like this:
                X X X X         X X X X         X X X X         X X X X
        (an X means the number matches the known bits, ' ' means it doesn't)

        Note that the lower and upper bounds could be more precise (ie bigger
        and smaller, respectively), because they don't match the known bits.
        Also, there are numbers that match the known bits to the left of lower
        and the right of upper, that are excluded by the range and thus
        unnecessary. We want to fix both of that.

        First we shrink the bounds so that they both match the known bits,
        then things look like this:
                                [lower   ...   upper]
                X X X X         X X X X         X X X X         X X X X
        This is achieved by moving lower to the right to the first number >=
        lower that matches the known bits (and mutatis mutandis for upper).

        Afterwards we use the information from the bounds to add more known
        bits. Having done that, things look maybe something like this:
                                [lower   ...   upper]
                                X X X X         X X X X
        Then we're done with shrinking.

        The set that is described by these two pieces of information together
        is neither expressible as purely a range, nor purely by known bits.
        It looks like this:
                                X X X X         X X X
        The set did not change through this process, but the bounds and the
        known bits becoming more precise makes it possible to compute more
        precise results when doing further operations with self.
        """
        # there's a proof in test_z3intbound that one pass of shrinking is
        # always enough. let's still assert that a second pass doesn't change
        # anything.
        changed = self._shrink_bounds_by_knownbits()
        changed |= self._shrink_knownbits_by_bounds()
        if not changed:
            return changed
        changed_again = self._shrink_bounds_by_knownbits()
        changed_again |= self._shrink_knownbits_by_bounds()
        assert not changed_again
        return changed

    def _shrink_bounds_by_knownbits(self):
        """
        Shrinks the bounds by the known bits.
        """
        # lower bound
        min_by_knownbits = self._get_minimum_signed_by_knownbits_atleast(self.lower)
        max_by_knownbits = self._get_maximum_signed_by_knownbits_atmost(self.upper)
        if min_by_knownbits > max_by_knownbits:
            raise InvalidLoop("range and knownbits contradict each other")
        changed = self.lower < min_by_knownbits or self.upper > max_by_knownbits
        if changed:
            self.lower = min_by_knownbits
            self.upper = max_by_knownbits
        return changed

    def _shrink_knownbits_by_bounds(self):
        """
        Infers known bits from the bounds. Basically fills a common prefix from
        lower and upper bound into the knownbits.
        """
        tvalue, tmask, valid = self._tnum_improve_knownbits_by_bounds()
        if not valid:
            raise InvalidLoop("knownbits and bounds don't agree")
        changed = self.tvalue != tvalue or self.tmask != tmask
        if changed:
            self.tmask = tmask
            self.tvalue = tvalue
        return changed

    @always_inline
    def _tnum_implied_by_bounds(self):
        # calculate higher bit mask by bounds
        hbm_bounds = leading_zeros_mask(
                self.r_uint(self.lower) ^ self.r_uint(self.upper))
        bounds_common = self.r_uint(self.lower) & hbm_bounds
        tmask = ~hbm_bounds
        return unmask_zero(bounds_common, tmask), tmask

    @always_inline
    def _tnum_improve_knownbits_by_bounds(self):
        # knownbits that are implied by the bounds
        tvalue, tmask = self._tnum_implied_by_bounds()
        # intersect them with the current knownbits
        return self._tnum_intersect(tvalue, tmask)

    def _debug_check(self):
        """
        Very simple debug check. Returns `True` iff the span of knownbits and
        the span of the bounds have a non-empty intersection. That does not
        guarantee for the actual concrete value set to contain any values!
        """
        assert self.lower <= self.upper
        min_knownbits = self._get_minimum_signed_by_knownbits()
        max_knownbits = self._get_maximum_signed_by_knownbits()
        if not min_knownbits <= self.upper:
            return False
        if not max_knownbits >= self.lower:
            return False
        # just to make sure
        if not min_knownbits <= max_knownbits:
            return False
        # make sure the set is not empty
        if self.is_constant():
            # does one constraint exclude the constant value?
            val = self.get_constant_int()
            if min_knownbits > val or max_knownbits < val \
               or self.lower > val or self.upper < val:
                return False
        else:
            # we have no constant, so keep checking
            u_lower = r_uint(self.lower)
            u_upper = r_uint(self.upper)
            # check if bounds common prefix agrees with known-bits
            hbm_bounds = leading_zeros_mask(u_lower ^ u_upper)
            bounds_common_prefix = u_lower & hbm_bounds
            if unmask_zero(bounds_common_prefix, self.tmask) != self.tvalue & hbm_bounds:
                return False
            # for the rest of the bunch, check by minima/maxima with threshold.
            #   (side note: the whole check can be reduced to this, but for the
            #    sake of robustness we want to keep the other checks above.)
            if self._get_minimum_signed_by_knownbits_atleast(self.lower) > self.upper \
               or self._get_maximum_signed_by_knownbits_atmost(self.upper) < self.lower:
                return False
        return True

    def knownbits_string(self, unk_sym='?'):
        """
        Returns a string representation about the knownbits part of this
        abstract integer. You can give any symbol or string for the "unknown
        bits" (default: '?'), the other digits are written as '1' and '0'.
        """
        results = []
        for bit in range(LONG_BIT):
            if self.tmask & (1 << bit):
                results.append(unk_sym)
            else:
                results.append(str((self.tvalue >> bit) & 1))
        results.reverse()
        return "".join(results)


def flip_msb(val_uint):
    return val_uint ^ r_uint(MININT)

def is_valid_tnum(tvalue, tmask):
    """
    Returns `True` iff `tvalue` and `tmask` would be valid tri-state number
    fields of an abstract integer, meeting all conventions and requirements.
    """
    if not isinstance(tvalue, r_uint):
        return False
    if not isinstance(tmask, r_uint):
        return False
    return 0 == (r_uint(tvalue) & r_uint(tmask))

def leading_zeros_mask(n):
    """
    calculates a bitmask in which only the leading zeros of `n` are set (1).
    """
    return ~next_pow2_m1(n)

def lowest_set_bit_only(val_uint):
    """
    Returns an val_int, but with all bits deleted but the lowest one that was
    set.
    """
    #assert isinstance(val_uint, r_uint)
    working_val = ~val_uint
    increased_val = working_val + 1
    result = (working_val^increased_val) & val_uint
    return result

def min4(t):
    """
    Returns the minimum of the values in the quadruplet t.
    """
    return min(min(t[0], t[1]), min(t[2], t[3]))

def max4(t):
    """
    Returns the maximum of the values in the quadruplet t.
    """
    return max(max(t[0], t[1]), max(t[2], t[3]))

def msbonly(v):
    """
    Returns `v` with all bits except the most significant bit set to 0 (zero).
    """
    return v & (1 << (LONG_BIT-1))

def next_pow2_m1(n):
    """Calculate next power of 2 minus one, greater than n"""
    n |= n >> 1
    n |= n >> 2
    n |= n >> 4
    n |= n >> 8
    n |= n >> 16
    if IS_64_BIT:
        n |= n >> 32
    return n

def unmask_one(value, mask):
    """
    Sets all unknowns determined by `mask` in `value` bit-wise to 1 (one) and
    returns the result.
    """
    return value | mask

def unmask_zero(value, mask):
    """
    Sets all unknowns determined by `mask` in `value` bit-wise to 0 (zero) and
    returns the result.
    """
    return value & ~mask

def saturating_mul(a, b):
    """
    Returns a * b. If the multiplication overflows it returns either MININT or
    MAXINT, whichever is closer to the "correct" value.
    """
    try:
        return ovfcheck(a * b)
    except OverflowError:
        if same_sign(a, b):
            return MAXINT
        return MININT

def same_sign(a, b):
    """ Return True iff a and b have the same sign """
    return (a ^ b) >= 0



from pypy.conftest import option
from pypy.module.micronumpy import constants as NPY
import platform


class BaseNumpyAppTest(object):
    spaceconfig = dict(usemodules=['micronumpy'])

    @classmethod
    def setup_class(cls):
        if option.runappdirect:
            import sys
            if '__pypy__' not in sys.builtin_module_names:
                import numpy
            else:
                from . import dummy_module as numpy
                sys.modules['numpy'] = numpy
        else:
            import os
            path = os.path.dirname(__file__) + '/dummy_module.py'
            cls.space.appexec([cls.space.wrap(path)], """(path):
            import imp
            imp.load_source('numpy', path)
            """)
        cls.w_non_native_prefix = cls.space.wrap(NPY.OPPBYTE)
        cls.w_native_prefix = cls.space.wrap(NPY.NATBYTE)
        cls.w_machine = cls.space.newtext(platform.machine())



from pypy.interpreter.module import Module
from pypy.interpreter.function import Function, BuiltinFunction
from pypy.interpreter import gateway
from pypy.interpreter.error import OperationError
from pypy.interpreter.baseobjspace import W_Root

from rpython.rlib.objectmodel import not_rpython

import sys

class MixedModule(Module):
    applevel_name = None

    # The following attribute is None as long as the module has not been
    # imported yet, and when it has been, it is mod.__dict__.items() just
    # after startup().
    w_initialdict = None
    lazy = False
    submodule_name = None

    @not_rpython
    def __init__(self, space, w_name):
        Module.__init__(self, space, w_name)
        self.lazy = True
        self.__class__.buildloaders()
        self.loaders = self.loaders.copy()    # copy from the class to the inst
        self.submodules_w = []

    @not_rpython
    def install(self):
        """Install this module, and its submodules into
        space.builtin_modules"""
        Module.install(self)
        if hasattr(self, "submodules"):
            space = self.space
            pkgname = space.text_w(self.w_name)
            for sub_name, module_cls in self.submodules.iteritems():
                if module_cls.submodule_name is None:
                    module_cls.submodule_name = sub_name
                else:
                    assert module_cls.submodule_name == sub_name
                name = "%s.%s" % (pkgname, sub_name)
                module_cls.applevel_name = name
                w_name = space.newtext(name)
                m = module_cls(space, w_name)
                m.install()
                self.submodules_w.append(m)

    def init(self, space):
        """This is called each time the module is imported or reloaded
        """
        if self.w_initialdict is not None:
            # the module was already imported.  Refresh its content with
            # the saved dict, as done with built-in and extension modules
            # on CPython.
            space.call_method(self.w_dict, 'update', self.w_initialdict)

        for w_submodule in self.submodules_w:
            name = space.text0_w(w_submodule.w_name)
            space.setitem(self.w_dict, space.newtext(name.split(".")[-1]), w_submodule)
            space.getbuiltinmodule(name)

        if self.w_initialdict is None:
            Module.init(self, space)
            if not self.lazy and self.w_initialdict is None:
                self.save_module_content_for_future_reload()

    def save_module_content_for_future_reload(self):
        self.w_initialdict = self.space.call_method(self.w_dict, 'items')

    @classmethod
    @not_rpython
    def get_applevel_name(cls):
        if cls.applevel_name is not None:
            return cls.applevel_name
        else:
            return cls.__module__.split('.')[-2]

    def get(self, name):
        space = self.space
        w_value = self.getdictvalue(space, name)
        if w_value is None:
            raise OperationError(space.w_AttributeError, space.newtext(name))
        return w_value

    def call(self, name, *args_w):
        w_builtin = self.get(name)
        return self.space.call_function(w_builtin, *args_w)

    def getdictvalue(self, space, name):
        w_value = space.finditem_str(self.w_dict, name)
        if self.lazy and w_value is None:
            return self._load_lazily(space, name)
        return w_value

    def _load_lazily(self, space, name):
        w_name = space.new_interned_str(name)
        try:
            loader = self.loaders[name]
        except KeyError:
            return None
        else:
            w_value = loader(space)
            # the idea of the following code is that all functions that are
            # directly in a mixed-module are "builtin", e.g. they get a
            # special type without a __get__
            # note that this is not just all functions that contain a
            # builtin code object, as e.g. methods of builtin types have to
            # be normal Functions to get the correct binding behaviour
            func = w_value
            if (isinstance(func, Function) and
                    type(func) is not BuiltinFunction):
                try:
                    bltin = func._builtinversion_
                except AttributeError:
                    bltin = BuiltinFunction(func)
                    bltin.w_module = self.w_name
                    func._builtinversion_ = bltin
                    bltin.name = name
                w_value = bltin
            space.setitem(self.w_dict, w_name, w_value)
            return w_value

    def getdict(self, space):
        if self.lazy:
            for name in self.loaders:
                w_value = self.get(name)
                space.setitem(self.w_dict, space.new_interned_str(name), w_value)
            self.lazy = False
            self.save_module_content_for_future_reload()
        return self.w_dict

    def _cleanup_(self):
        self.getdict(self.space)
        self.w_initialdict = None
        self.startup_called = False
        self._frozen = True

    @classmethod
    @not_rpython
    def buildloaders(cls):
        if not hasattr(cls, 'loaders'):
            # build a constant dictionary out of
            # applevel/interplevel definitions
            cls.loaders = loaders = {}
            pkgroot = cls.__module__.rsplit('.', 1)[0]
            appname = cls.get_applevel_name()
            for name, spec in cls.interpleveldefs.items():
                loaders[name] = getinterpevalloader(pkgroot, spec)
            for name, spec in cls.appleveldefs.items():
                loaders[name] = getappfileloader(pkgroot, appname, spec)
            assert '__file__' not in loaders
            if '__doc__' not in loaders:
                loaders['__doc__'] = cls.get__doc__

    def extra_interpdef(self, name, spec):
        cls = self.__class__
        pkgroot = cls.__module__.rsplit('.', 1)[0]
        loader = getinterpevalloader(pkgroot, spec)
        space = self.space
        w_obj = loader(space)
        space.setattr(self, space.newtext(name), w_obj)

    @classmethod
    def get__doc__(cls, space):
        return space.newtext_or_none(cls.__doc__)


@not_rpython
def getinterpevalloader(pkgroot, spec):
    def ifileloader(space):
        d = {'space':space}
        # EVIL HACK (but it works, and this is not RPython :-)
        while 1:
            try:
                value = eval(spec, d)
            except NameError as ex:
                name = ex.args[0].split("'")[1]  # super-Evil
                if name in d:
                    raise   # propagate the NameError
                try:
                    d[name] = __import__(pkgroot+'.'+name, None, None, [name])
                except ImportError:
                    etype, evalue, etb = sys.exc_info()
                    try:
                        d[name] = __import__(name, None, None, [name])
                    except ImportError:
                        # didn't help, re-raise the original exception for
                        # clarity
                        raise etype, evalue, etb
            else:
                #print spec, "->", value
                if hasattr(value, 'func_code'):  # semi-evil
                    return gateway.interp2app(value).get_function(space)

                try:
                    is_type = issubclass(value, W_Root)  # pseudo-evil
                except TypeError:
                    is_type = False
                if is_type:
                    return space.gettypefor(value)

                assert isinstance(value, W_Root), (
                    "interpleveldef %s.%s must return a wrapped object "
                    "(got %r instead)" % (pkgroot, spec, value))
                return value
    return ifileloader

applevelcache = {}
@not_rpython
def getappfileloader(pkgroot, appname, spec):
    # hum, it's a bit more involved, because we usually
    # want the import at applevel
    modname, attrname = spec.split('.')
    impbase = pkgroot + '.' + modname
    try:
        app = applevelcache[impbase]
    except KeyError:
        import imp
        pkg = __import__(pkgroot, None, None, ['__doc__'])
        file, fn, (suffix, mode, typ) = imp.find_module(modname, pkg.__path__)
        assert typ == imp.PY_SOURCE
        source = file.read()
        file.close()
        if fn.endswith('.pyc') or fn.endswith('.pyo'):
            fn = fn[:-1]
        app = gateway.applevel(source, filename=fn, modname=appname)
        applevelcache[impbase] = app

    def afileloader(space):
        return app.wget(space, attrname)
    return afileloader



from pypy.module.pypyjit.test_pypy_c.test_00_model import BaseTestPyPyC


class TestBuffers(BaseTestPyPyC):
    def test_re_match(self):
        def main(n):
            import re
            import array
            p = re.compile('.+')
            a = array.array('c', 'test' * 1000)
            i = 0
            while i < n:
                i += 1
                p.match(a)  # ID: match
            return i
        log = self.run(main, [1000])
        assert log.result == 1000
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match_by_id('match', """
            guard_not_invalidated(descr=...)
            i65 = getfield_gc_i(p18, descr=...)
            i67 = int_gt(0, i65)
            guard_false(i67, descr=...)
            i69 = int_gt(#, i65)
            guard_true(i69, descr=...)
            --TICK--
        """)

    def test_struct_unpack(self):
        def main(n):
            import _struct as struct
            import array
            a = array.array('c', struct.pack('i', 42))
            i = 0
            while i < n:
                i += 1
                struct.unpack('i', a)  # ID: unpack
            return i
        log = self.run(main, [1000])
        assert log.result == 1000
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match_by_id('unpack', """
            guard_not_invalidated(descr=...)
            i66 = raw_load_i(i53, 0, descr=<ArrayS 4>)
            --TICK--
        """)



import py, pytest, sys, textwrap
from inspect import isclass

APPLEVEL_FN = 'apptest_*.py'

# pytest settings
rsyncdirs = ['.', '../lib-python', '../lib_pypy', '../demo']
rsyncignore = ['_cache']

try:
    from hypothesis import settings, __version__
except ImportError:
    pass
else:
    try:
        settings.register_profile('default', deadline=None)
    except Exception:
        import warnings
        warnings.warn("Version of hypothesis too old, "
                      "cannot set the deadline to None")
    settings.load_profile('default')

# PyPy's command line extra options (these are added
# to py.test's standard options)
#
option = None

def braindead_deindent(self):
    """monkeypatch that wont end up doing stupid in the python tokenizer"""
    text = '\n'.join(self.lines)
    short = py.std.textwrap.dedent(text)
    newsource = py.code.Source()
    newsource.lines[:] = short.splitlines()
    return newsource

py.code.Source.deindent = braindead_deindent

def get_marker(item, name):
    try:
        return item.get_closest_marker(name=name)
    except AttributeError:
        # pytest < 3.6
        return item.get_marker(name=name)

def pytest_report_header():
    return "pytest-%s from %s" % (pytest.__version__, pytest.__file__)

@pytest.hookimpl(tryfirst=True)
def pytest_cmdline_preparse(config, args):
    if not (set(args) & {'-D', '--direct-apptest'}):
        args.append('--assert=reinterp')

def pytest_configure(config):
    global option
    option = config.option
    mode_A = config.getoption('runappdirect')
    mode_D = config.getoption('direct_apptest')
    if mode_D or not mode_A:
        config.addinivalue_line('python_files', APPLEVEL_FN)
    if not mode_A and not mode_D:  # 'own' tests
        from rpython.conftest import LeakFinder
        config.pluginmanager.register(LeakFinder())
    config.addinivalue_line("markers", "flaky: test that sometimes fails")


def pytest_addoption(parser):
    group = parser.getgroup("pypy options")
    group.addoption('-A', '--runappdirect', action="store_true",
           default=False, dest="runappdirect",
           help="run legacy applevel tests directly on python interpreter (not through PyPy)")
    group.addoption('-D', '--direct-apptest', action="store_true",
           default=False, dest="direct_apptest",
           help="run applevel_XXX.py tests directly on host interpreter")
    group.addoption('--direct', action="store_true",
           default=False, dest="rundirect",
           help="run pexpect tests directly")
    group.addoption('--raise-operr', action="store_true",
            default=False, dest="raise_operr",
            help="Show the interp-level OperationError in app-level tests")
    group.addoption('--no-applevel-rewrite', action="store_false",
            default=True, dest="applevel_rewrite",
            help="Don't use assert rewriting in app-level test files")

@pytest.fixture(scope='class')
def spaceconfig(request):
    return getattr(request.cls, 'spaceconfig', {})

@pytest.fixture(scope='function')
def space(spaceconfig):
    from pypy.tool.pytest.objspace import gettestobjspace
    return gettestobjspace(**spaceconfig)


#
# Interfacing/Integrating with py.test's collection process
#
#

def ensure_pytest_builtin_helpers(helpers='skip raises'.split()):
    """ hack (py.test.) raises and skip into builtins, needed
        for applevel tests to run directly on cpython but
        apparently earlier on "raises" was already added
        to module's globals.
    """
    import __builtin__
    for helper in helpers:
        if not hasattr(__builtin__, helper):
            setattr(__builtin__, helper, getattr(py.test, helper))

def pytest_sessionstart(session):
    """ before session.main() is called. """
    # stick py.test raise in module globals -- carefully
    ensure_pytest_builtin_helpers()

def pytest_pycollect_makemodule(path, parent):
    if path.fnmatch(APPLEVEL_FN):
        if parent.config.getoption('direct_apptest'):
            return
        from pypy.tool.pytest.apptest2 import AppTestModule
        rewrite = parent.config.getoption('applevel_rewrite')
        return AppTestModule(path, parent, rewrite_asserts=rewrite)
    else:
        return PyPyModule(path, parent)

def is_applevel(item):
    from pypy.tool.pytest.apptest import AppTestMethod
    return isinstance(item, AppTestMethod)

def pytest_collection_modifyitems(config, items):
    if config.getoption('runappdirect') or config.getoption('direct_apptest'):
        return
    for item in items:
        if isinstance(item, py.test.Function):
            if is_applevel(item):
                item.add_marker('applevel')
            else:
                item.add_marker('interplevel')


class PyPyModule(pytest.Module):
    """ we take care of collecting classes both at app level
        and at interp-level (because we need to stick a space
        at the class) ourselves.
    """
    def accept_regular_test(self):
        if self.config.option.runappdirect:
            # only collect regular tests if we are in test_lib_pypy
            for name in self.listnames():
                if "test_lib_pypy" in name:
                    return True
            return False
        return True

    def funcnamefilter(self, name):
        if name.startswith('test_'):
            return self.accept_regular_test()
        return False

    def classnamefilter(self, name):
        if name.startswith('Test'):
            return self.accept_regular_test()
        if name.startswith('AppTest'):
            return True
        return False

    def makeitem(self, name, obj):
        if isclass(obj) and self.classnamefilter(name):
            if name.startswith('AppTest'):
                from pypy.tool.pytest.apptest import AppClassCollector
                return AppClassCollector(name, parent=self)
        return super(PyPyModule, self).makeitem(name, obj)

def skip_on_missing_buildoption(**ropts):
    __tracebackhide__ = True
    import sys
    options = getattr(sys, 'pypy_translation_info', None)
    if options is None:
        py.test.skip("not running on translated pypy "
                     "(btw, i would need options: %s)" %
                     (ropts,))
    for opt in ropts:
        if not options.has_key(opt) or options[opt] != ropts[opt]:
            break
    else:
        return
    py.test.skip("need translated pypy with: %s, got %s"
                 %(ropts,options))

@pytest.hookimpl(tryfirst=True)
def pytest_runtest_setup(item):
    if isinstance(item, py.test.collect.Function):
        config = item.config
        appdirect = (config.getoption('runappdirect') or
            config.getoption('direct_apptest'))
        if (get_marker(item, name='pypy_only') and
                appdirect and not '__pypy__' in sys.builtin_module_names):
            pytest.skip('PyPy-specific test')
        appclass = item.getparent(py.test.Class)
        if appclass is not None:
            from pypy.tool.pytest.objspace import gettestobjspace
            # Make cls.space and cls.runappdirect available in tests.
            spaceconfig = getattr(appclass.obj, 'spaceconfig', {})
            if not appdirect:
                spaceconfig.setdefault('objspace.std.reinterpretasserts', True)
            appclass.obj.space = gettestobjspace(**spaceconfig)
            appclass.obj.runappdirect = config.option.runappdirect

def pytest_ignore_collect(path, config):
    if (config.getoption('direct_apptest') and not path.isdir()
            and not path.fnmatch(APPLEVEL_FN)):
        return True
    return path.check(link=1)



from __future__ import with_statement
from rpython.rlib import rfloat
from rpython.translator.tool.cbuild import ExternalCompilationInfo
from rpython.translator import cdir
from rpython.rtyper.lltypesystem import lltype, rffi
from rpython.rlib import jit, objectmodel
from rpython.rlib.rstring import StringBuilder
import py, sys

cdir = py.path.local(cdir)
include_dirs = [cdir]

# set the word endianness based on the host's endianness
# and the C double's endianness (which should be equal)
if hasattr(float, '__getformat__'):
    assert float.__getformat__('double') == 'IEEE, %s-endian' % sys.byteorder
if sys.byteorder == 'little':
    source_file = ['#define DOUBLE_IS_LITTLE_ENDIAN_IEEE754']
elif sys.byteorder == 'big':
    source_file = ['#define WORDS_BIGENDIAN',
                   '#define DOUBLE_IS_BIG_ENDIAN_IEEE754']
else:
    raise AssertionError(sys.byteorder)

source_file.append('#include "src/dtoa.c"')
source_file = '\n\n'.join(source_file)

# ____________________________________________________________

eci = ExternalCompilationInfo(
    include_dirs = [cdir],
    includes = ['src/dtoa.h'],
    libraries = [],
    separate_module_sources = [source_file],
    )

# dtoa.c is limited to 'int', so we refuse to pass it
# strings or integer arguments bigger than ~2GB
_INT_LIMIT = 0x7ffff000

dg_strtod = rffi.llexternal(
    '_PyPy_dg_strtod', [rffi.CONST_CCHARP, rffi.CCHARPP], rffi.DOUBLE,
    compilation_info=eci, sandboxsafe=True)

dg_dtoa = rffi.llexternal(
    '_PyPy_dg_dtoa', [rffi.DOUBLE, rffi.INT, rffi.INT,
                    rffi.INT_realP, rffi.INT_realP, rffi.CCHARPP], rffi.CCHARP,
    compilation_info=eci, sandboxsafe=True)

dg_freedtoa = rffi.llexternal(
    '_PyPy_dg_freedtoa', [rffi.CCHARP], lltype.Void,
    compilation_info=eci, sandboxsafe=True)

def strtod(input):
    if len(input) > _INT_LIMIT:
        raise MemoryError
    if objectmodel.revdb_flag_io_disabled():
        return _revdb_strtod(input)
    end_ptr = lltype.malloc(rffi.CCHARPP.TO, 1, flavor='raw')
    try:
        # note: don't use the class scoped_view_charp here, it
        # break some tests because this function is used by the GC
        ll_input, llobj, flag = rffi.get_nonmovingbuffer_ll_final_null(input)
        try:
            result = dg_strtod(rffi.cast(rffi.CONST_CCHARP, ll_input), end_ptr)

            endpos = (rffi.cast(lltype.Signed, end_ptr[0]) -
                      rffi.cast(lltype.Signed, ll_input))
        finally:
            rffi.free_nonmovingbuffer_ll(ll_input, llobj, flag)
    finally:
        lltype.free(end_ptr, flavor='raw')

    if endpos == 0 or endpos < len(input):
        raise ValueError("invalid input at position %d" % (endpos,))

    return result

lower_special_strings = ['inf', '+inf', '-inf', 'nan']
upper_special_strings = ['INF', '+INF', '-INF', 'NAN']

def format_nonfinite(digits, sign, flags, special_strings):
    "Format dtoa's output for nonfinite numbers"
    if digits[0] == 'i' or digits[0] == 'I':
        if sign == 1:
            return special_strings[2]
        elif flags & rfloat.DTSF_SIGN:
            return special_strings[1]
        else:
            return special_strings[0]
    elif digits[0] == 'n' or digits[0] == 'N':
        return special_strings[3]
    else:
        # shouldn't get here
        raise ValueError

@jit.dont_look_inside
def format_number(digits, buflen, sign, decpt, code, precision, flags, upper):
    # We got digits back, format them.  We may need to pad 'digits'
    # either on the left or right (or both) with extra zeros, so in
    # general the resulting string has the form
    #
    # [<sign>]<zeros><digits><zeros>[<exponent>]
    #
    # where either of the <zeros> pieces could be empty, and there's a
    # decimal point that could appear either in <digits> or in the
    # leading or trailing <zeros>.
    #
    # Imagine an infinite 'virtual' string vdigits, consisting of the
    # string 'digits' (starting at index 0) padded on both the left
    # and right with infinite strings of zeros.  We want to output a
    # slice
    #
    # vdigits[vdigits_start : vdigits_end]
    #
    # of this virtual string.  Thus if vdigits_start < 0 then we'll
    # end up producing some leading zeros; if vdigits_end > digits_len
    # there will be trailing zeros in the output.  The next section of
    # code determines whether to use an exponent or not, figures out
    # the position 'decpt' of the decimal point, and computes
    # 'vdigits_start' and 'vdigits_end'.
    builder = StringBuilder(20)

    use_exp = False
    vdigits_end = buflen
    if code == 'e':
        use_exp = True
        vdigits_end = precision
    elif code == 'f':
        vdigits_end = decpt + precision
    elif code == 'g':
        if decpt <= -4:
            use_exp = True
        elif decpt > precision:
            use_exp = True
        elif flags & rfloat.DTSF_ADD_DOT_0 and decpt == precision:
            use_exp = True
        if flags & rfloat.DTSF_ALT:
            vdigits_end = precision
    elif code == 'r':
        #  convert to exponential format at 1e16.  We used to convert
        #  at 1e17, but that gives odd-looking results for some values
        #  when a 16-digit 'shortest' repr is padded with bogus zeros.
        #  For example, repr(2e16+8) would give 20000000000000010.0;
        #  the true value is 20000000000000008.0.
        if decpt <= -4 or decpt > 16:
            use_exp = True
    else:
        raise ValueError

    # if using an exponent, reset decimal point position to 1 and
    # adjust exponent accordingly.
    if use_exp:
        exp = decpt - 1
        decpt = 1
    else:
        exp = 0

    # ensure vdigits_start < decpt <= vdigits_end, or vdigits_start <
    # decpt < vdigits_end if add_dot_0_if_integer and no exponent
    if decpt <= 0:
        vdigits_start = decpt-1
    else:
        vdigits_start = 0
    if vdigits_end <= decpt:
        if not use_exp and flags & rfloat.DTSF_ADD_DOT_0:
            vdigits_end = decpt + 1
        else:
            vdigits_end = decpt

    # double check inequalities
    assert vdigits_start <= 0
    assert 0 <= buflen <= vdigits_end
    # decimal point should be in (vdigits_start, vdigits_end]
    assert vdigits_start < decpt <= vdigits_end

    if sign == 1:
        builder.append('-')
    elif flags & rfloat.DTSF_SIGN:
        builder.append('+')

    # note that exactly one of the three 'if' conditions is true, so
    # we include exactly one decimal point
    # 1. Zero padding on left of digit string
    if decpt <= 0:
        builder.append_multiple_char('0', decpt - vdigits_start)
        builder.append('.')
        builder.append_multiple_char('0', 0 - decpt)
    else:
        builder.append_multiple_char('0', 0 - vdigits_start)

    # 2. Digits, with included decimal point
    if 0 < decpt <= buflen:
        builder.append_charpsize(digits, decpt - 0)
        builder.append('.')
        ptr = rffi.ptradd(digits, decpt)
        builder.append_charpsize(ptr, buflen - decpt)
    else:
        builder.append_charpsize(digits, buflen)

    # 3. And zeros on the right
    if buflen < decpt:
        builder.append_multiple_char('0', decpt - buflen)
        builder.append('.')
        builder.append_multiple_char('0', vdigits_end - decpt)
    else:
        builder.append_multiple_char('0', vdigits_end - buflen)

    s = builder.build()

    # Delete a trailing decimal pt unless using alternative formatting.
    if not flags & rfloat.DTSF_ALT:
        last = len(s) - 1
        if last >= 0 and s[last] == '.':
            s = s[:last]

    # remove the - for negative zeros if DTSF_NO_NEG_0 is given
    if flags & rfloat.DTSF_NO_NEG_0 and s and s[0] == '-':
        for index in range(1, len(s)):
            c = s[index]
            if c != '.' and c != '0':
                break
        else:
            s = s[1:]

    # Now that we've done zero padding, add an exponent if needed.
    if use_exp:
        if upper:
            e = 'E'
        else:
            e = 'e'

        if exp >= 0:
            exp_str = str(exp)
            if len(exp_str) < 2 and not (flags & rfloat.DTSF_CUT_EXP_0):
                s += e + '+0' + exp_str
            else:
                s += e + '+' + exp_str
        else:
            exp_str = str(-exp)
            if len(exp_str) < 2 and not (flags & rfloat.DTSF_CUT_EXP_0):
                s += e + '-0' + exp_str
            else:
                s += e + '-' + exp_str

    return s

@jit.elidable
def dtoa(value, code='r', mode=0, precision=0, flags=0,
         special_strings=lower_special_strings, upper=False):
    if precision > _INT_LIMIT:
        raise MemoryError
    if objectmodel.revdb_flag_io_disabled():
        return _revdb_dtoa(value)
    decpt_ptr = lltype.malloc(rffi.INT_realP.TO, 1, flavor='raw')
    try:
        sign_ptr = lltype.malloc(rffi.INT_realP.TO, 1, flavor='raw')
        try:
            end_ptr = lltype.malloc(rffi.CCHARPP.TO, 1, flavor='raw')
            try:
                digits = dg_dtoa(value, mode, precision,
                                     decpt_ptr, sign_ptr, end_ptr)
                if not digits:
                    # The only failure mode is no memory
                    raise MemoryError
                try:
                    buflen = (rffi.cast(lltype.Signed, end_ptr[0]) -
                              rffi.cast(lltype.Signed, digits))
                    sign = rffi.cast(lltype.Signed, sign_ptr[0])

                    # Handle nan and inf
                    if buflen and not digits[0].isdigit():
                        return format_nonfinite(digits, sign, flags,
                                                special_strings)

                    decpt = rffi.cast(lltype.Signed, decpt_ptr[0])

                    return format_number(digits, buflen, sign, decpt,
                                         code, precision, flags, upper)

                finally:
                    dg_freedtoa(digits)
            finally:
                lltype.free(end_ptr, flavor='raw')
        finally:
            lltype.free(sign_ptr, flavor='raw')
    finally:
        lltype.free(decpt_ptr, flavor='raw')

def dtoa_formatd(value, code, precision, flags):
    if code in 'EFG':
        code = code.lower()
        special_strings = upper_special_strings
        upper = True
    else:
        special_strings = lower_special_strings
        upper = False

    if code == 'e':
        mode = 2
        precision += 1
    elif code == 'f':
        mode = 3
    elif code == 'g':
        mode = 2
        # precision 0 makes no sense for 'g' format; interpret as 1
        if precision == 0:
            precision = 1
    elif code == 'r':
        # repr format
        mode = 0
        assert precision == 0
    else:
        raise ValueError('Invalid mode')

    return dtoa(value, code, mode=mode, precision=precision, flags=flags,
                special_strings=special_strings, upper=upper)

def _revdb_strtod(input):
    # moved in its own function for the import statement
    from rpython.rlib import revdb
    return revdb.emulate_strtod(input)

def _revdb_dtoa(value):
    # moved in its own function for the import statement
    from rpython.rlib import revdb
    return revdb.emulate_dtoa(value)



import sys
from rpython.rtyper.lltypesystem import lltype, llmemory, rstr
from rpython.rtyper.lltypesystem.lloperation import llop
from rpython.rtyper.annlowlevel import llhelper
from rpython.jit.backend.llsupport import jitframe, gc, descr
from rpython.jit.backend.llsupport import symbolic
from rpython.jit.metainterp.gc import get_description
from rpython.jit.metainterp.history import ConstPtr
from rpython.jit.metainterp.resoperation import get_deep_immutable_oplist, rop,\
     ResOperation, InputArgRef
from rpython.rlib.rarithmetic import is_valid_int, r_uint

def test_boehm():
    gc_ll_descr = gc.GcLLDescr_boehm(None, None, None)
    #
    record = []
    prev_malloc_fn_ptr = gc_ll_descr.malloc_fn_ptr
    def my_malloc_fn_ptr(size):
        p = prev_malloc_fn_ptr(size)
        record.append((size, p))
        return p
    gc_ll_descr.malloc_fn_ptr = my_malloc_fn_ptr
    #
    # ---------- gc_malloc ----------
    S = lltype.GcStruct('S', ('x', lltype.Signed))
    sizedescr = descr.get_size_descr(gc_ll_descr, S)
    p = gc_ll_descr.gc_malloc(sizedescr)
    assert record == [(sizedescr.size, p)]
    del record[:]
    # ---------- gc_malloc_array ----------
    A = lltype.GcArray(lltype.Signed)
    arraydescr = descr.get_array_descr(gc_ll_descr, A)
    p = gc_ll_descr.gc_malloc_array(10, arraydescr)
    assert record == [(arraydescr.basesize +
                       10 * arraydescr.itemsize, p)]
    del record[:]
    # ---------- gc_malloc_str ----------
    p = gc_ll_descr.gc_malloc_str(10)
    basesize, itemsize, ofs_length = symbolic.get_array_token(rstr.STR, False)
    assert record == [(basesize + 10 * itemsize, p)]
    del record[:]
    # ---------- gc_malloc_unicode ----------
    p = gc_ll_descr.gc_malloc_unicode(10)
    basesize, itemsize, ofs_length = symbolic.get_array_token(rstr.UNICODE,
                                                              False)
    assert record == [(basesize + 10 * itemsize, p)]
    del record[:]

# ____________________________________________________________


class FakeLLOp(object):
    def __init__(self):
        self.record = []

    def _malloc(self, type_id, size):
        tid = llop.combine_ushort(lltype.Signed, type_id, 0)
        x = llmemory.raw_malloc(self.gcheaderbuilder.size_gc_header + size)
        x += self.gcheaderbuilder.size_gc_header
        return x, tid

    def do_malloc_fixedsize(self, RESTYPE, type_id, size,
                                  has_finalizer, has_light_finalizer,
                                  contains_weakptr):
        assert not contains_weakptr
        assert not has_finalizer
        assert not has_light_finalizer
        p, tid = self._malloc(type_id, size)
        p = llmemory.cast_adr_to_ptr(p, RESTYPE)
        self.record.append(("fixedsize", repr(size), tid, p))
        return p

    do_malloc_fixedsize_clear = do_malloc_fixedsize

    def do_malloc_varsize(self, RESTYPE, type_id, length, size,
                                itemsize, offset_to_length):
        p, tid = self._malloc(type_id, size + itemsize * length)
        (p + offset_to_length).signed[0] = length
        p = llmemory.cast_adr_to_ptr(p, RESTYPE)
        self.record.append(("varsize", tid, length,
                            repr(size), repr(itemsize),
                            repr(offset_to_length), p))
        return p

    do_malloc_varsize_clear = do_malloc_varsize

    def _write_barrier_failing_case(self, adr_struct):
        self.record.append(('barrier', adr_struct))

    def get_write_barrier_failing_case(self, FPTRTYPE):
        return llhelper(FPTRTYPE, self._write_barrier_failing_case)

    _have_wb_from_array = False

    def _write_barrier_from_array_failing_case(self, adr_struct, v_index):
        self.record.append(('barrier_from_array', adr_struct, v_index))

    def get_write_barrier_from_array_failing_case(self, FPTRTYPE):
        if self._have_wb_from_array:
            return llhelper(FPTRTYPE,
                            self._write_barrier_from_array_failing_case)
        else:
            return lltype.nullptr(FPTRTYPE.TO)


class TestFramework(object):
    gc = 'minimark'

    def setup_method(self, meth):
        class config_(object):
            class translation(object):
                gc = self.gc
                gcrootfinder = 'shadowstack'
                gctransformer = 'framework'
                gcremovetypeptr = False
        class FakeTranslator(object):
            config = config_
        class FakeCPU(object):
            def cast_adr_to_int(self, adr):
                if not adr:
                    return 0
                try:
                    ptr = llmemory.cast_adr_to_ptr(adr, gc_ll_descr.WB_FUNCPTR)
                    assert ptr._obj._callable == \
                           llop1._write_barrier_failing_case
                    return 42
                except lltype.InvalidCast:
                    ptr = llmemory.cast_adr_to_ptr(
                        adr, gc_ll_descr.WB_ARRAY_FUNCPTR)
                    assert ptr._obj._callable == \
                           llop1._write_barrier_from_array_failing_case
                    return 43

        gcdescr = get_description(config_)
        llop1 = FakeLLOp()
        gc_ll_descr = gc.GcLLDescr_framework(gcdescr, FakeTranslator(), None,
                                             llop1)
        gc_ll_descr.initialize()
        llop1.gcheaderbuilder = gc_ll_descr.gcheaderbuilder
        self.llop1 = llop1
        self.gc_ll_descr = gc_ll_descr
        self.fake_cpu = FakeCPU()

    def test_gc_malloc(self):
        S = lltype.GcStruct('S', ('x', lltype.Signed))
        sizedescr = descr.get_size_descr(self.gc_ll_descr, S)
        p = self.gc_ll_descr.gc_malloc(sizedescr)
        assert lltype.typeOf(p) == llmemory.GCREF
        assert self.llop1.record == [("fixedsize", repr(sizedescr.size),
                                      sizedescr.tid, p)]

    def test_gc_malloc_array(self):
        A = lltype.GcArray(lltype.Signed)
        arraydescr = descr.get_array_descr(self.gc_ll_descr, A)
        p = self.gc_ll_descr.gc_malloc_array(10, arraydescr)
        assert self.llop1.record == [("varsize", arraydescr.tid, 10,
                                      repr(arraydescr.basesize),
                                      repr(arraydescr.itemsize),
                                      repr(arraydescr.lendescr.offset),
                                      p)]

    def test_gc_malloc_str(self):
        p = self.gc_ll_descr.gc_malloc_str(10)
        type_id = self.gc_ll_descr.layoutbuilder.get_type_id(rstr.STR)
        tid = llop.combine_ushort(lltype.Signed, type_id, 0)
        basesize, itemsize, ofs_length = symbolic.get_array_token(rstr.STR,
                                                                  True)
        assert self.llop1.record == [("varsize", tid, 10,
                                      repr(basesize), repr(itemsize),
                                      repr(ofs_length), p)]

    def test_gc_malloc_unicode(self):
        p = self.gc_ll_descr.gc_malloc_unicode(10)
        type_id = self.gc_ll_descr.layoutbuilder.get_type_id(rstr.UNICODE)
        tid = llop.combine_ushort(lltype.Signed, type_id, 0)
        basesize, itemsize, ofs_length = symbolic.get_array_token(rstr.UNICODE,
                                                                  True)
        assert self.llop1.record == [("varsize", tid, 10,
                                      repr(basesize), repr(itemsize),
                                      repr(ofs_length), p)]

    def test_gen_write_barrier(self):
        gc_ll_descr = self.gc_ll_descr
        llop1 = self.llop1
        #
        rewriter = gc.GcRewriterAssembler(gc_ll_descr, None)
        newops = rewriter._newops
        v_base = InputArgRef()
        rewriter.gen_write_barrier(v_base)
        assert llop1.record == []
        assert len(newops) == 1
        assert newops[0].getopnum() == rop.COND_CALL_GC_WB
        assert newops[0].getarg(0) == v_base
        wbdescr = newops[0].getdescr()
        assert is_valid_int(wbdescr.jit_wb_if_flag)
        assert is_valid_int(wbdescr.jit_wb_if_flag_byteofs)
        assert is_valid_int(wbdescr.jit_wb_if_flag_singlebyte)


class TestFrameworkMiniMark(TestFramework):
    gc = 'minimark'

def test_custom_tracer():
    def indexof(no):
        return (frame_adr + jitframe.getofs('jf_frame') +
                jitframe.BASEITEMOFS + jitframe.SIGN_SIZE * no)

    frame_info = lltype.malloc(jitframe.JITFRAMEINFO, zero=True, flavor='raw')
    frame = lltype.malloc(jitframe.JITFRAME, 200, zero=True)
    frame.jf_frame_info = frame_info
    frame.jf_gcmap = lltype.malloc(jitframe.GCMAP, 4, flavor='raw')
    if sys.maxint == 2**31 - 1:
        max = r_uint(2 ** 31)
    else:
        max = r_uint(2 ** 63)
    frame.jf_gcmap[0] = r_uint(1 | 2 | 8 | 32 | 128) | max
    frame.jf_gcmap[1] = r_uint(0)
    frame.jf_gcmap[2] = r_uint(2 | 16 | 32 | 128)
    frame.jf_gcmap[3] = r_uint(0)
    frame_adr = llmemory.cast_ptr_to_adr(frame)
    #
    all_addrs = []
    class FakeGC:
        def _trace_callback(self, callback, arg1, arg2, addr):
            assert callback == "hello"
            assert arg1 == "world"
            assert arg2 == "foo"
            all_addrs.append(addr)
    jitframe.jitframe_trace(FakeGC(), frame_adr, "hello", "world", "foo")
    #
    counter = 0
    for name in jitframe.JITFRAME._names:
        TP = getattr(jitframe.JITFRAME, name)
        if isinstance(TP, lltype.Ptr) and TP.TO._gckind == 'gc':
            assert all_addrs[counter] == frame_adr + jitframe.getofs(name)
            counter += 1
    assert counter == 5
    # gcpattern
    assert all_addrs[5] == indexof(0)
    assert all_addrs[6] == indexof(1)
    assert all_addrs[7] == indexof(3)
    assert all_addrs[8] == indexof(5)
    assert all_addrs[9] == indexof(7)
    if sys.maxint == 2**31 - 1:
        assert all_addrs[10] == indexof(31)
        assert all_addrs[11] == indexof(65)
        assert all_addrs[12] == indexof(68)
        assert all_addrs[13] == indexof(69)
        assert all_addrs[14] == indexof(71)
    else:
        assert all_addrs[10] == indexof(63)
        assert all_addrs[11] == indexof(129)
        assert all_addrs[12] == indexof(132)
        assert all_addrs[13] == indexof(133)
        assert all_addrs[14] == indexof(135)

    assert len(all_addrs) == 15
    lltype.free(frame_info, flavor='raw')
    lltype.free(frame.jf_gcmap, flavor='raw')

def test_custom_tracer_2():
    frame_info = lltype.malloc(jitframe.JITFRAMEINFO, zero=True, flavor='raw')
    frame = lltype.malloc(jitframe.JITFRAME, 200, zero=True)
    frame.jf_frame_info = frame_info
    frame.jf_gcmap = lltype.malloc(jitframe.GCMAP, 3, flavor='raw')
    frame.jf_gcmap[0] = r_uint(18446744073441116160)
    frame.jf_gcmap[1] = r_uint(18446740775107559407)
    frame.jf_gcmap[2] = r_uint(3)
    frame_adr = llmemory.cast_ptr_to_adr(frame)
    class FakeGC:
        def _trace_callback(self, callback, arg1, arg2, addr):
            assert callback == "hello"
            assert arg1 == "world"
            assert arg2 == "foo"
    jitframe.jitframe_trace(FakeGC(), frame_adr, "hello", "world", "foo")
    # assert did not hang

    lltype.free(frame_info, flavor='raw')
    lltype.free(frame.jf_gcmap, flavor='raw')



from rpython.jit.backend.ppc.form import Form
from rpython.jit.backend.ppc.ppc_field import ppc_fields

class PPCForm(Form):
    fieldmap = ppc_fields

    def __init__(self, *fnames):
        super(PPCForm, self).__init__(*("opcode",) + fnames)

    def __call__(self, opcode, **specializations):
        specializations['opcode'] = opcode
        return super(PPCForm, self).__call__(**specializations)



import random
import py
from rpython.jit.backend.arm.registers import *
from rpython.jit.backend.arm.locations import *
from rpython.jit.backend.arm.regalloc import ARMFrameManager
from rpython.jit.backend.arm.jump import remap_frame_layout, remap_frame_layout_mixed
from rpython.jit.metainterp.history import INT

fm = ARMFrameManager(0)
frame_pos = fm.frame_pos

class MockAssembler:
    def __init__(self):
        self.ops = []

    def regalloc_mov(self, from_loc, to_loc):
        self.ops.append(('mov', from_loc, to_loc))

    def regalloc_push(self, loc):
        self.ops.append(('push', loc))

    def regalloc_pop(self, loc):
        self.ops.append(('pop', loc))

    def got(self, expected):
        print '------------------------ comparing ---------------------------'
        for op1, op2 in zip(self.ops, expected):
            print '%-38s| %-38s' % (op1, op2)
            if op1 == op2:
                continue
            assert len(op1) == len(op2)
            for x, y in zip(op1, op2):
                if isinstance(x, StackLoc) and isinstance(y, MODRM):
                    assert x.byte == y.byte
                    assert x.extradata == y.extradata
                else:
                    assert x == y
        assert len(self.ops) == len(expected)
        return True

class TestJump(object):
    def setup_method(self, m):
        self.assembler = MockAssembler()

    def test_trivial(self):
        remap_frame_layout(self.assembler, [], [], '?')
        assert self.assembler.ops == []
        remap_frame_layout(self.assembler, [r0, r1, r3, r5, r6, r7, r9],
                                      [r0, r1, r3, r5, r6, r7, r9], '?')
        assert self.assembler.ops == []
        s8 = frame_pos(1, INT)
        s12 = frame_pos(31, INT)
        s20 = frame_pos(6, INT)
        remap_frame_layout(self.assembler, [r0, r1, s20, s8, r3, r5, r6, s12, r7, r9],
                                      [r0, r1, s20, s8, r3, r5, r6, s12, r7, r9],
                                      '?')
        assert self.assembler.ops == []

    def test_simple_registers(self):
        remap_frame_layout(self.assembler, [r0, r1, r2], [r3, r4, r5], '?')
        assert self.assembler.ops == [('mov', r0, r3),
                                 ('mov', r1, r4),
                                 ('mov', r2, r5)]

    def test_simple_framelocs(self):
        s8 = frame_pos(0, INT)
        s12 = frame_pos(13, INT)
        s20 = frame_pos(20, INT)
        s24 = frame_pos(221, INT)
        remap_frame_layout(self.assembler, [s8, r7, s12], [s20, s24, r9], ip)
        assert self.assembler.ops == [('mov', s8, ip),
                                 ('mov', ip, s20),
                                 ('mov', r7, s24),
                                 ('mov', s12, r9)]

    def test_reordering(self):
        s8 = frame_pos(8, INT)
        s12 = frame_pos(12, INT)
        s20 = frame_pos(19, INT)
        s24 = frame_pos(1, INT)
        remap_frame_layout(self.assembler, [r7, s8, s20, r4],
                                      [s8, r4, r7, r2], '?')
        assert self.assembler.got([('mov', r4, r2),
                              ('mov', s8, r4),
                              ('mov', r7, s8),
                              ('mov', s20, r7)])

    def test_cycle(self):
        s8 = frame_pos(8, INT)
        s12 = frame_pos(12, INT)
        s20 = frame_pos(19, INT)
        s24 = frame_pos(1, INT)
        remap_frame_layout(self.assembler, [r4, s8, s20, r7],
                                      [s8, r7, r4, s20], '?')
        assert self.assembler.got([('push', s8),
                              ('mov', r4, s8),
                              ('mov', s20, r4),
                              ('mov', r7, s20),
                              ('pop', r7)])

    def test_cycle_2(self):
        s8 = frame_pos(8, INT)
        s12 = frame_pos(12, INT)
        s20 = frame_pos(19, INT)
        s24 = frame_pos(1, INT)
        s2 = frame_pos(2, INT)
        s3 = frame_pos(3, INT)
        remap_frame_layout(self.assembler,
                           [r0, s8, r1, s20, r0, s20, s24, r3, s2, s3],
                           [s8, s20, r1, r0, r4, s24, r5, s12, s3, s2],
                           ip)
        assert self.assembler.got([('mov', r0, r4),
                              ('mov', s24, r5),
                              ('mov', r3, s12),
                              ('mov', s20, ip),
                              ('mov', ip, s24),
                              ('push', s8),
                              ('mov', r0, s8),
                              ('mov', s20, r0),
                              ('pop', s20),
                              ('push', s3),
                              ('mov', s2, ip),
                              ('mov', ip, s3),
                              ('pop', s2)])

    def test_constants(self):
        c3 = ImmLocation(3)
        remap_frame_layout(self.assembler, [c3], [r0], '?')
        assert self.assembler.ops == [('mov', c3, r0)]

    def test_constants2(self):
        c3 = ImmLocation(3)
        s12 = frame_pos(12, INT)
        remap_frame_layout(self.assembler, [c3], [s12], '?')
        assert self.assembler.ops == [('mov', c3, s12)]

    def test_constants_and_cycle(self):
        c3 = ImmLocation(3)
        s12 = frame_pos(13, INT)
        remap_frame_layout(self.assembler, [r5, c3,  s12],
                                      [s12, r0, r5], r1)
        assert self.assembler.ops == [('mov', c3, r0),
                                 ('push', s12),
                                 ('mov', r5, s12),
                                 ('pop', r5)]
    def test_mixed(self):
        s23 = frame_pos(2, FLOAT)     # non-conflicting locations
        s4  = frame_pos(4, INT)
        remap_frame_layout_mixed(self.assembler, [r1], [s4], 'tmp',
                                            [s23], [d5], 'vfptmp')
        assert self.assembler.ops == [('mov', r1, s4),
                                 ('mov', s23, d5)]
    def test_mixed2(self):
        s23 = frame_pos(2, FLOAT)  # gets stored in pos 2 and 3, with value==3
        s3  = frame_pos(3, INT)
        remap_frame_layout_mixed(self.assembler, [r1], [s3], 'tmp',
                                            [s23], [d5], 'vfptmp')
        assert self.assembler.ops == [('push', s23),
                                 ('mov', r1, s3),
                                 ('pop', d5)]
    def test_mixed3(self):
        s23 = frame_pos(2, FLOAT)
        s2  = frame_pos(2, INT)
        remap_frame_layout_mixed(self.assembler, [r1], [s2], 'tmp',
                                            [s23], [d5], 'vfptmp')
        assert self.assembler.ops == [
                                 ('push', s23),
                                 ('mov', r1, s2),
                                 ('pop', d5)]
    def test_mixed4(self):
        s23 = frame_pos(2, FLOAT)
        s4  = frame_pos(4, INT)
        s45 = frame_pos(4, FLOAT)
        s1  = frame_pos(1, INT)
        remap_frame_layout_mixed(self.assembler, [s4], [s1], r3,
                                            [s23], [s45], d3)
        assert self.assembler.ops == [('mov', s4, r3),
                                 ('mov', r3, s1),
                                 ('mov', s23, d3),
                                 ('mov', d3, s45)]
    def test_mixed5(self):
        s2  = frame_pos(2, INT)
        s23 = frame_pos(2, FLOAT)
        s4  = frame_pos(4, INT)
        s45 = frame_pos(4, FLOAT)
        remap_frame_layout_mixed(self.assembler, [s4], [s2], r3,
                                            [s23], [s45], d3)
        assert self.assembler.ops == [('push', s23),
                                 ('mov', s4, r3),
                                 ('mov', r3, s2),
                                 ('pop', s45)]
    def test_mixed6(self):
        s3  = frame_pos(3, INT)
        s23 = frame_pos(2, FLOAT)
        s4  = frame_pos(4, INT)
        s45 = frame_pos(4, FLOAT)
        remap_frame_layout_mixed(self.assembler, [s4], [s3], r3,
                                     [s23], [s45], d3)
        assert self.assembler.ops == [('push', s23),
                                     ('mov', s4, r3),
                                     ('mov', r3, s3),
                                     ('pop', s45)]

def test_random_mixed():
    assembler = MockAssembler()
    registers1 = all_regs
    registers2 = all_vfp_regs
    VFPWORDS = 2
    #
    def pick1():
        n = random.randrange(-3, 10)
        if n < 0:
            return registers1[n]
        else:
            return frame_pos(n, INT)
    def pick2():
        n = random.randrange(-3 , 10 // VFPWORDS)
        if n < 0:
            return registers2[n]
        else:
            return frame_pos(n*VFPWORDS, FLOAT)
    #
    def pick1c():
        n = random.randrange(-2000, 500)
        if n >= 0:
            return imm(n)
        else:
            return pick1()
    #
    def pick_dst(fn, count, seen):
        result = []
        while len(result) < count:
            x = fn()
            keys = [x.as_key()]
            if x.is_stack() and x.width > WORD:
                keys.append(keys[0] + 1)
            for key in keys:
                if key in seen:
                    break
            else:
                for key in keys:
                    seen[key] = True
                result.append(x)
        return result
    #
    def get_state(locations):
        regs1 = {}
        regs2 = {}
        stack = {}
        for i, loc in enumerate(locations):
            if loc.is_vfp_reg():
                if loc.width > WORD:
                    newvalue = ('value-vfp-%d' % i,
                                'value-vfp-hiword-%d' % i)
                else:
                    newvalue = 'value-vfp-%d' % i
                regs2[loc.value] = newvalue
            elif loc.is_core_reg():
                regs1[loc.value] = 'value-int-%d' % i
            elif loc.is_stack():
                stack[loc.position] = 'value-width%d-%d' % (loc.width, i)
                if loc.width > WORD:
                    stack[loc.position+1] = 'value-hiword-%d' % i
            else:
                assert loc.is_imm() or loc.is_imm_float()
        return regs1, regs2, stack
    #
    for i in range(500):
        seen = {}
        src_locations2 = [pick2() for i in range(4)]
        dst_locations2 = pick_dst(pick2, 4, seen)
        src_locations1 = [pick1c() for i in range(5)]
        dst_locations1 = pick_dst(pick1, 5, seen)
        assembler = MockAssembler()
        remap_frame_layout_mixed(assembler,
                                 src_locations1, dst_locations1, ip,
                                 src_locations2, dst_locations2, vfp_ip)
        #
        regs1, regs2, stack = get_state(src_locations1 +
                                        src_locations2)
        #
        def read(loc, expected_width=None):
            if expected_width is not None:
                assert loc.width == expected_width*WORD
            if loc.is_vfp_reg():
                return regs2[loc.value]
            elif loc.is_core_reg():
                return regs1[loc.value]
            elif loc.is_stack():
                got = stack[loc.position]
                if loc.width > WORD:
                    got = (got, stack[loc.position+1])
                return got
            if loc.is_imm() or loc.is_imm_float():
                return 'const-%d' % loc.value
            assert 0, loc
        #
        def write(loc, newvalue):
            if loc.is_vfp_reg():
                regs2[loc.value] = newvalue
            elif loc.is_core_reg():
                regs1[loc.value] = newvalue
            elif loc.is_stack():
                if loc.width > WORD:
                    newval1, newval2 = newvalue
                    stack[loc.position] = newval1
                    stack[loc.position+1] = newval2
                else:
                    stack[loc.position] = newvalue
            else:
                assert 0, loc
        #
        src_values1 = [read(loc, 1) for loc in src_locations1]
        src_values2 = [read(loc, 2)    for loc in src_locations2]
        #
        extrapushes = []
        for op in assembler.ops:
            if op[0] == 'mov':
                src, dst = op[1:]
                assert src.is_core_reg() or src.is_vfp_reg() or src.is_stack() or src.is_imm_float() or src.is_imm()
                assert dst.is_core_reg() or dst.is_vfp_reg() or dst.is_stack()
                assert not (src.is_stack() and dst.is_stack())
                write(dst, read(src))
            elif op[0] == 'push':
                src, = op[1:]
                assert src.is_core_reg() or src.is_vfp_reg() or src.is_stack()
                extrapushes.append(read(src))
            elif op[0] == 'pop':
                dst, = op[1:]
                assert dst.is_core_reg() or dst.is_vfp_reg() or dst.is_stack()
                write(dst, extrapushes.pop())
            else:
                assert 0, "unknown op: %r" % (op,)
        assert not extrapushes
        #
        for i, loc in enumerate(dst_locations1):
            assert read(loc, 1) == src_values1[i]
        for i, loc in enumerate(dst_locations2):
            assert read(loc, 2) == src_values2[i]




""" Storage format:
for each operation (inputargs numbered with negative numbers)
<opnum> [size-if-unknown-arity] [<arg0> <arg1> ...] [descr-or-snapshot-index]

<opnum> is 1 byte
size is 1 byte (or 0 if implied by opnum)
the args are varsized
the index is varsized

Snapshot index for guards are an index into the _snapshot_data byte array.
Snapshots reference arrays of (encoded) boxes, the data of which is stored
in _snapshot_array_data.

Top snapshots start with:
    vable_array_index
    vref_array_index 
    ... then continue like a normal snapshot

regular snapshots:
    jitcode_index or -1 for empty snapshots
    pc
    array_index
    prev

prev:
    an index into _snapshot_data
    SNAPSHOT_PREV_NONE for the very final snapshot
    SNAPSHOT_PREV_COMES_NEXT if the prev snapshot comes right *after* the
    current one
"""

from rpython.jit.metainterp.history import (
    ConstInt, Const, ConstFloat, ConstPtr, new_ref_dict, SwitchToBlackhole,
    ConstPtrJitCode)
from rpython.jit.metainterp.resoperation import AbstractResOp, AbstractInputArg,\
    ResOperation, oparity, rop, opwithdescr, GuardResOp, IntOp, FloatOp, RefOp,\
    opclasses
from rpython.rlib.rarithmetic import intmask, r_uint
from rpython.rlib.objectmodel import we_are_translated, specialize, always_inline
from rpython.rlib.jit import Counters
from rpython.rtyper.lltypesystem import rffi, lltype, llmemory
from rpython.rlib.debug import make_sure_not_resized

TAGINT, TAGCONSTPTR, TAGCONSTOTHER, TAGBOX = range(4)
TAGMASK = 0x3
TAGSHIFT = 2

INIT_SIZE = 4096

# XXX todos left:
# - SnapshotIterator is very inefficient
# - fix test_resume
# - benchmarks

MIN_VALUE = -2 ** 30
MAX_VALUE = 2 ** 30 - 1

def encode_varint_signed(i, res):
    # either encode in 2 bytes or 4 bytes. the topmost bit is not set in the
    # 2-bytes case
    startlen = len(res)
    assert MIN_VALUE <= i <= MAX_VALUE
    flag = bool(not (-2**14 <= i < 2 ** 14)) << 7
    res.append(chr((i & 0b1111111) | flag))
    i >>= 7
    res.append(chr(i & 0xff))
    if flag:
        i >>= 8
        res.append(chr(i & 0xff))
        i >>= 8
        res.append(chr(i & 0xff))
    return len(res) - startlen

@always_inline
def decode_varint_signed(b, index=0):
    byte = ord(b[index])
    lastbyte = ord(b[index + 1])
    res = (byte & 0b1111111) | (lastbyte << 7)
    shift = 15
    index += 2
    if byte & 0b10000000:
        lastbyte = ord(b[index + 1])
        res |= (ord(b[index]) << 15) | (lastbyte << 23)
        shift = 31
        index += 2
    # sign-extend
    res |= (-bool(lastbyte & 0b10000000)) << shift
    return res, index

def skip_varint_signed(b, index, skip=1):
    assert skip > 0
    while True:
        byte = ord(b[index])
        if byte & 0b10000000:
            index += 2
        index += 2
        skip -= 1
        if not skip:
            return index

def varint_only_decode(b, index, skip=0):
    if skip:
        index = skip_varint_signed(b, index, skip)
    byte = ord(b[index])
    lastbyte = ord(b[index + 1])
    res = (byte & 0b1111111) | (lastbyte << 7)
    shift = 15
    index += 2
    if byte & 0b10000000:
        lastbyte = ord(b[index + 1])
        res |= (ord(b[index]) << 15) | (lastbyte << 23)
        shift = 31
        index += 2
    # sign-extend
    res |= (-bool(lastbyte & 0b10000000)) << shift
    return res

def tag(kind, pos):
    res = intmask(r_uint(pos) << TAGSHIFT)
    assert res >> TAGSHIFT == pos
    return res | kind

@specialize.ll()
def untag(tagged):
    return intmask(tagged) & TAGMASK, intmask(tagged) >> TAGSHIFT

# chosen such that constant ints fit into the four bytes
SMALL_INT_STOP  = 2**28
SMALL_INT_START = -2**28
assert encode_varint_signed(tag(TAGINT, SMALL_INT_START), []) <= 4
assert encode_varint_signed(tag(TAGINT, SMALL_INT_STOP - 1), []) <= 4

class BaseTrace(object):
    pass

SNAPSHOT_PREV_NEEDS_PATCHING = -3
SNAPSHOT_PREV_NONE = -2
SNAPSHOT_PREV_COMES_NEXT = -1

class TopDownSnapshotIterator(object):
    """ iterates over encoded snapshots, but in top-down order, ie the most
    recent call frame is returned last. more efficient, because that's the
    order than things are encoded in."""

    def __init__(self, trace, snapshot_index):
        self.trace = trace

        self._index = snapshot_index
        self.vable_array_index = self.decode_snapshot_int()
        self.vref_array_index = self.decode_snapshot_int()
        self.snapshot_index = self._index

    def iter_vable_array(self):
        return BoxArrayIter.make(self.vable_array_index, self.trace._snapshot_array_data)

    def iter_vref_array(self):
        return BoxArrayIter.make(self.vref_array_index, self.trace._snapshot_array_data)

    def iter_array(self, snapshot_index):
        array = varint_only_decode(self.trace._snapshot_data, snapshot_index, skip=2)
        return BoxArrayIter.make(array, self.trace._snapshot_array_data)

    def length(self, snapshot_index):
        array = varint_only_decode(self.trace._snapshot_data, snapshot_index, skip=2)
        length = varint_only_decode(self.trace._snapshot_array_data, array)
        return length

    def prev(self, snapshot_index):
        self._index = skip_varint_signed(self.trace._snapshot_data, snapshot_index, skip=3)
        prev = self.decode_snapshot_int()
        assert prev != SNAPSHOT_PREV_NEEDS_PATCHING
        if prev == SNAPSHOT_PREV_COMES_NEXT:
            prev = self._index
        return prev

    def unpack_jitcode_pc(self, snapshot_index):
        self._index = snapshot_index
        jitcode_index = self.decode_snapshot_int()
        pc = self.decode_snapshot_int()
        return jitcode_index, pc

    def is_empty_snapshot(self, snapshot_index):
        # must be a top snapshot. it's empty if the jitcode index is -1
        return varint_only_decode(self.trace._snapshot_data, snapshot_index, skip=2) == -1

    def decode_snapshot_int(self):
        result, self._index = decode_varint_signed(self.trace._snapshot_data, self._index)
        return result

    def __iter__(self):
        return self

    def next(self):
        res = self.snapshot_index
        if res == SNAPSHOT_PREV_NONE:
            raise StopIteration
        self.snapshot_index = self.prev(res)
        return res


class SnapshotIterator(object):
    def __init__(self, main_iter, snapshot_index):
        self.main_iter = main_iter
        # reverse the snapshots and store the vable, vref lists
        it = TopDownSnapshotIterator(main_iter.trace, snapshot_index)
        self.topdown_snapshot_iter = it
        self.vable_array = it.iter_vable_array()
        self.vref_array = it.iter_vref_array()
        self.size = self.vable_array.total_length + self.vref_array.total_length + 3
        self.framestack = []
        if it.is_empty_snapshot(snapshot_index):
            return
        for snapshot_index in it:
            self.framestack.append(snapshot_index)
            self.size += it.length(snapshot_index) + 2
        self.framestack.reverse()

    def iter_array(self, snapshot_index):
        return self.topdown_snapshot_iter.iter_array(snapshot_index)

    def get(self, index):
        return self.main_iter._untag(index)

    def unpack_jitcode_pc(self, snapshot_index):
        return self.topdown_snapshot_iter.unpack_jitcode_pc(snapshot_index)

    def unpack_array(self, arr):
        # NOT_RPYTHON
        # for tests only
        assert isinstance(arr, BoxArrayIter)
        return [self.get(i) for i in arr]

def _update_liverange(item, index, liveranges):
    tag, v = untag(item)
    if tag == TAGBOX:
        liveranges[v] = index

def update_liveranges(snapshot_index, trace, index, liveranges):
    it = TopDownSnapshotIterator(trace, snapshot_index)
    for item in it.iter_vable_array():
        _update_liverange(item, index, liveranges)
    for item in it.iter_vref_array():
        _update_liverange(item, index, liveranges)
    for snapshot_index in it:
        for item in it.iter_array(snapshot_index):
            _update_liverange(item, index, liveranges)

class TraceIterator(BaseTrace):
    def __init__(self, trace, start, end, force_inputargs=None,
                 metainterp_sd=None):
        self.trace = trace
        self.metainterp_sd = metainterp_sd
        self.all_descr_len = len(metainterp_sd.all_descrs)
        self._cache = [None] * trace._index
        if force_inputargs is not None:
            # the trace here is cut and we're working from
            # inputargs that are in the middle, shuffle stuff around a bit
            self.inputargs = [rop.inputarg_from_tp(arg.type) for
                              arg in force_inputargs]
            for i, arg in enumerate(force_inputargs):
                self._cache[arg.get_position()] = self.inputargs[i]
        else:
            self.inputargs = [rop.inputarg_from_tp(arg.type) for
                              arg in self.trace.inputargs]
            for i, arg in enumerate(self.inputargs):
               self._cache[self.trace.inputargs[i].get_position()] = arg
        self.start = start
        self.pos = start
        self._count = start
        self._index = start
        self.start_index = start
        self.end = end

    def get_dead_ranges(self):
        return self.trace.get_dead_ranges()

    def kill_cache_at(self, pos):
        if pos:
            self._cache[pos] = None

    def replace_last_cached(self, oldbox, box):
        assert self._cache[self._index - 1] is oldbox
        self._cache[self._index - 1] = box

    def _get(self, i):
        res = self._cache[i]
        assert res is not None
        return res

    def done(self):
        return self.pos >= self.end

    def _nextbyte(self):
        if self.done():
            raise IndexError
        res = ord(self.trace._ops[self.pos])
        self.pos += 1
        return res
        
    def _next(self):
        if self.done():
            raise IndexError
        b = self.trace._ops
        index = self.pos
        byte = ord(b[index])
        lastbyte = ord(b[index + 1])
        res = (byte & 0b1111111) | (lastbyte << 7)
        shift = 15
        index += 2
        if byte & 0b10000000:
            lastbyte = ord(b[index + 1])
            res |= (ord(b[index]) << 15) | (lastbyte << 23)
            shift = 31
            index += 2
        # sign-extend
        res |= (-bool(lastbyte & 0b10000000)) << shift
        self.pos = index
        return res

    def _untag(self, tagged):
        tag, v = untag(tagged)
        if tag == TAGBOX:
            return self._get(v)
        elif tag == TAGINT:
            return ConstInt(v)
        elif tag == TAGCONSTPTR:
            return ConstPtr(self.trace._refs[v])
        elif tag == TAGCONSTOTHER:
            if v & 1:
                return ConstFloat(self.trace._floats[v >> 1])
            else:
                return ConstInt(self.trace._bigints[v >> 1])
        else:
            assert False

    def get_snapshot_iter(self, index):
        return SnapshotIterator(self, index)

    def next_element_update_live_range(self, index, liveranges):
        opnum = self._nextbyte()
        if oparity[opnum] == -1:
            argnum = self._next()
        else:
            argnum = oparity[opnum]
        for i in range(argnum):
            tagged = self._next()
            tag, v = untag(tagged)
            if tag == TAGBOX:
                liveranges[v] = index
        if opclasses[opnum].type != 'v':
            liveranges[index] = index
        if opwithdescr[opnum]:
            descr_index = self._next()
            if rop.is_guard(opnum):
                update_liveranges(descr_index, self.trace, index,
                                  liveranges)
        if opclasses[opnum].type != 'v':
            return index + 1
        return index

    def next(self):
        opnum = self._nextbyte()
        argnum = oparity[opnum]
        if argnum == -1:
            argnum = self._next()
        if not (0 <= oparity[opnum] <= 3):
            args = [None] * argnum
            for i in range(argnum):
                args[i] = self._untag(self._next())
            res = ResOperation(opnum, args)
        else:
            cls = opclasses[opnum]
            res = cls()
            argnum = oparity[opnum]
            if argnum == 0:
                pass
            elif argnum == 1:
                res.setarg(0, self._untag(self._next()))
            elif argnum == 2:
                res.setarg(0, self._untag(self._next()))
                res.setarg(1, self._untag(self._next()))
            else:
                assert argnum == 3
                res.setarg(0, self._untag(self._next()))
                res.setarg(1, self._untag(self._next()))
                res.setarg(2, self._untag(self._next()))
        descr_index = -1
        if opwithdescr[opnum]:
            descr_index = self._next()
            if descr_index == 0 or rop.is_guard(opnum):
                descr = None
            else:
                if descr_index < self.all_descr_len + 1:
                    descr = self.metainterp_sd.all_descrs[descr_index - 1]
                else:
                    descr = self.trace._descrs[descr_index - self.all_descr_len - 1]
                res.setdescr(descr)
            if rop.is_guard(opnum): # all guards have descrs
                assert isinstance(res, GuardResOp)
                res.rd_resume_position = descr_index
        if res.type != 'v':
            self._cache[self._index] = res
            self._index += 1
        self._count += 1
        return res

class CutTrace(BaseTrace):
    def __init__(self, trace, start, count, index, inputargs):
        self.trace = trace
        self.start = start
        self.inputargs = inputargs
        self.count = count
        self.index = index

    def cut_at(self, cut):
        assert cut[1] > self.count
        self.trace.cut_at(cut)

    def get_iter(self):
        iter = TraceIterator(self.trace, self.start, self.trace._pos,
                             self.inputargs,
                             metainterp_sd=self.trace.metainterp_sd)
        iter._count = self.count
        iter.start_index = self.index
        iter._index = self.index
        return iter

def combine_uint(index1, index2):
    assert 0 <= index1 < 65536
    assert 0 <= index2 < 65536
    return index1 << 16 | index2 # it's ok to return signed here,
    # we need only 32bit, but 64 is ok for now

def unpack_uint(packed):
    return (packed >> 16) & 0xffff, packed & 0xffff

class BoxArrayIter(object):
    def __init__(self, index, data):
        self.length, self.position = decode_varint_signed(data, index)
        self.total_length = self.length
        self.data = data

    def __repr__(self):
        if self.length == 0 and self.data == ['\x00', '\x00']:
            return "BoxArrayIter.BOXARRAYITER0"
        return "<BoxArrayIter length=%s>" % self.length

    @staticmethod
    def make(index, data):
        if index == 0: # length 0 arrays are always encoded to index 0
            return BoxArrayIter.BOXARRAYITER0
        return BoxArrayIter(index, data)

    def __iter__(self):
        return self

    def next(self):
        if self.length == 0:
            raise StopIteration
        self.length -= 1
        item, self.position = decode_varint_signed(self.data, self.position)
        return item

BoxArrayIter.BOXARRAYITER0 = BoxArrayIter(0, ['\x00', '\x00'])


class Trace(BaseTrace):
    _deadranges = (-1, None)

    def __init__(self, max_num_inputargs, metainterp_sd):
        self.metainterp_sd = metainterp_sd
        self._ops = ['\x00'] * INIT_SIZE
        make_sure_not_resized(self._ops)
        self._pos = 0
        self._consts_bigint = 0
        self._consts_float = 0
        self._total_snapshots = 0
        self._consts_ptr = 0
        self._consts_ptr_nodict = 0
        self._descrs = [None]
        self._refs = [lltype.nullptr(llmemory.GCREF.TO)]
        self._refs_dict = new_ref_dict()
        self._bigints = []
        self._bigints_dict = {}
        self._floats = []
        self._snapshot_data = []
        self._snapshot_array_data = []
        self.append_snapshot_array_data_int(0) # all 0-length arrays get index 0
        if not we_are_translated() and isinstance(max_num_inputargs, list): # old api for tests
            self.inputargs = max_num_inputargs
            for i, box in enumerate(max_num_inputargs):
                box.position_and_flags = r_uint(i << 1)
            max_num_inputargs = len(max_num_inputargs)

        self.max_num_inputargs = max_num_inputargs
        self._count = max_num_inputargs # total count
        self._index = max_num_inputargs # "position" of resulting resops
        self._start = max_num_inputargs
        self._pos = max_num_inputargs
        self.tag_overflow = False

    def set_inputargs(self, inputargs):
        self.inputargs = inputargs
        if not we_are_translated():
            set_positions = {box.get_position() for box in inputargs}
            assert len(set_positions) == len(inputargs)
            assert not set_positions or max(set_positions) < self.max_num_inputargs

    def _double_ops(self):
        self._ops = self._ops + ['\x00'] * len(self._ops)

    def append_byte(self, c):
        assert 0 <= c < 256
        if self._pos >= len(self._ops):
            self._double_ops()
        self._ops[self._pos] = chr(c)
        self._pos += 1

    def append_int(self, i):
        if not MIN_VALUE <= i <= MAX_VALUE:
            self.tag_overflow = True
            i = 0
        if self._pos + 4 > len(self._ops):
            self._double_ops()
        ops = self._ops
        pos = self._pos
        flag = bool(not (-2**14 <= i < 2 ** 14)) << 7
        ops[pos] = chr((i & 0b1111111) | flag)
        pos += 1
        i >>= 7
        ops[pos] = chr(i & 0xff)
        pos += 1
        if flag:
            i >>= 8
            ops[pos] = chr(i & 0xff)
            pos += 1
            i >>= 8
            ops[pos] = chr(i & 0xff)
            pos += 1
        self._pos = pos

    def tag_overflow_imminent(self):
        return self._pos > MAX_VALUE * 0.8

    def tracing_done(self):
        from rpython.rlib.debug import debug_start, debug_stop, debug_print
        if self.tag_overflow:
            raise SwitchToBlackhole(Counters.ABORT_TOO_LONG)
        self._bigints_dict = {}
        self._refs_dict = new_ref_dict()
        debug_start("jit-trace-done")
        debug_print("trace length:", self._pos)
        debug_print(" number operations:", self._count)
        debug_print(" total snapshots:", self._total_snapshots)
        debug_print(" snapshot data:", len(self._snapshot_data))
        debug_print(" snapshot array data:", len(self._snapshot_array_data))
        debug_print(" bigint consts: ", self._consts_bigint, len(self._bigints))
        debug_print(" float consts: ", self._consts_float, len(self._floats))
        debug_print(" ref consts: ", self._consts_ptr, self._consts_ptr_nodict,  len(self._refs))
        debug_print(" descrs:", len(self._descrs))
        debug_stop("jit-trace-done")

    def length(self):
        return self._pos

    def cut_point(self):
        return self._pos, self._count, self._index, len(self._snapshot_data), len(self._snapshot_array_data)

    def cut_at(self, end):
        self._pos = end[0]
        self._count = end[1]
        index = end[2]
        assert index >= 0
        self._index = index

    def cut_trace_from(self, (start, count, index, x, y), inputargs):
        return CutTrace(self, start, count, index, inputargs)

    def _cached_const_int(self, box):
        return v

    def _cached_const_ptr(self, box):
        assert isinstance(box, ConstPtr)
        addr = box.getref_base()
        if not addr:
            return 0
        if isinstance(box, ConstPtrJitCode):
            index = box.opencoder_index
            if index >= 0:
                self._consts_ptr_nodict += 1
                assert self._refs[index] == addr
                return index
        v = self._refs_dict.get(addr, -1)
        if v == -1:
            v = len(self._refs)
            self._refs_dict[addr] = v
            self._refs.append(addr)
        if isinstance(box, ConstPtrJitCode):
            box.opencoder_index = v
        return v

    def _encode(self, box):
        if isinstance(box, Const):
            if (isinstance(box, ConstInt) and
                isinstance(box.getint(), int) and # symbolics
                SMALL_INT_START <= box.getint() < SMALL_INT_STOP):
                return tag(TAGINT, box.getint())
            elif isinstance(box, ConstInt):
                self._consts_bigint += 1
                value = box.getint()
                if not isinstance(value, int):
                    # symbolics, for tests, don't worry about caching
                    v = len(self._bigints) << 1
                    self._bigints.append(value)
                else:
                    v = self._bigints_dict.get(value, -1)
                    if v == -1:
                        v = len(self._bigints) << 1
                        self._bigints_dict[value] = v
                        self._bigints.append(value)
                return tag(TAGCONSTOTHER, v)
            elif isinstance(box, ConstFloat):
                # don't intern float constants
                self._consts_float += 1
                v = (len(self._floats) << 1) | 1
                self._floats.append(box.getfloatstorage())
                return tag(TAGCONSTOTHER, v)
            else:
                self._consts_ptr += 1
                v = self._cached_const_ptr(box)
                return tag(TAGCONSTPTR, v)
        elif isinstance(box, AbstractResOp):
            position = box.get_position()
            assert position >= 0
            # every time something is used we assume that it lives to the
            # current _index
            return tag(TAGBOX, position)
        else:
            assert False, "unreachable code"

    def _op_start(self, opnum, num_argboxes):
        old_pos = self._pos
        self.append_byte(opnum)
        expected_arity = oparity[opnum]
        if expected_arity == -1:
            self.append_int(num_argboxes)
        else:
            assert num_argboxes == expected_arity
        return old_pos

    def _op_end(self, opnum, descr, old_pos):
        if opwithdescr[opnum]:
            if descr is None:
                # guards get a 0 which is later patched. also some tests have
                # missing descrs
                self.append_int(0)
            else:
                self.append_int(self._encode_descr(descr))
        self._count += 1
        if opclasses[opnum].type != 'v':
            self._index += 1

    def record_op(self, opnum, argboxes, descr=None):
        pos = self._index
        old_pos = self._op_start(opnum, len(argboxes))
        for box in argboxes:
            self.append_int(self._encode(box))
        self._op_end(opnum, descr, old_pos)
        return pos

    def record_op0(self, opnum, descr=None):
        pos = self._index
        old_pos = self._op_start(opnum, 0)
        self._op_end(opnum, descr, old_pos)
        return pos

    def record_op1(self, opnum, argbox1, descr=None):
        pos = self._index
        old_pos = self._op_start(opnum, 1)
        self.append_int(self._encode(argbox1))
        self._op_end(opnum, descr, old_pos)
        return pos

    def record_op2(self, opnum, argbox1, argbox2, descr=None):
        pos = self._index
        old_pos = self._op_start(opnum, 2)
        self.append_int(self._encode(argbox1))
        self.append_int(self._encode(argbox2))
        self._op_end(opnum, descr, old_pos)
        return pos

    def record_op3(self, opnum, argbox1, argbox2, argbox3, descr=None):
        pos = self._index
        old_pos = self._op_start(opnum, 3)
        self.append_int(self._encode(argbox1))
        self.append_int(self._encode(argbox2))
        self.append_int(self._encode(argbox3))
        self._op_end(opnum, descr, old_pos)
        return pos

    def _encode_descr(self, descr):
        descr_index = descr.get_descr_index()
        if descr_index != -1:
            return descr_index + 1
        self._descrs.append(descr)
        return len(self._descrs) - 1 + len(self.metainterp_sd.all_descrs) + 1

    # ____________________________________________________________
    # snapshots

    def _list_of_boxes(self, boxes):
        boxes_list_storage = self.new_array(len(boxes))
        for i in range(len(boxes)):
            self._add_box_to_storage(boxes[i])
        return boxes_list_storage

    def _list_of_boxes_virtualizable(self, boxes):
        if not boxes:
            return self.new_array(0)
        boxes_list_storage = self.new_array(len(boxes))
        # the virtualizable is at the end, move it to the front in the snapshot
        self._add_box_to_storage(boxes[-1])
        for i in range(len(boxes) - 1):
            self._add_box_to_storage(boxes[i])
        return boxes_list_storage

    def new_array(self, lgt):
        if lgt == 0:
            return 0
        res = len(self._snapshot_array_data)
        self.append_snapshot_array_data_int(lgt)
        return res

    def _add_box_to_storage(self, box):
        self.append_snapshot_array_data_int(self._encode(box))

    def append_snapshot_array_data_int(self, i):
        if not MIN_VALUE <= i <= MAX_VALUE:
            self.tag_overflow = True
            i = 0
        encode_varint_signed(i, self._snapshot_array_data)

    def append_snapshot_data_int(self, i):
        if not MIN_VALUE <= i <= MAX_VALUE:
            self.tag_overflow = True
            i = 0
        encode_varint_signed(i, self._snapshot_data)

    def _encode_snapshot(self, index, pc, array, is_last=False):
        res = len(self._snapshot_data)
        self.append_snapshot_data_int(index)
        self.append_snapshot_data_int(pc)
        # arrays are indexes into self._snapshot_data
        self.append_snapshot_data_int(array)
        # for prev. can be SNAPSHOT_PREV_COMES_NEXT, which means the prev
        # snapshot comes right afterwards in snapshots. or it can be
        # SNAPSHOT_PREV_NONE, which means there are no more snapshots. or it's
        # a positive number, which means it's already somewhere in snapshot at
        # some other index
        if is_last:
            self.append_snapshot_data_int(SNAPSHOT_PREV_NONE)
        else:
            self.append_snapshot_data_int(SNAPSHOT_PREV_NEEDS_PATCHING)
        return res

    def create_top_snapshot(self, frame, vable_boxes, vref_boxes, after_residual_call=False, is_last=False):
        self._total_snapshots += 1
        array = frame.get_list_of_active_boxes(False, self.new_array, self._add_box_to_storage,
                after_residual_call=after_residual_call)
        s = len(self._snapshot_data)
        vable_array = self._list_of_boxes_virtualizable(vable_boxes)
        vref_array = self._list_of_boxes(vref_boxes)
        self.append_snapshot_data_int(vable_array)
        self.append_snapshot_data_int(vref_array)
        self._encode_snapshot(
                frame.jitcode.index,
                frame.pc,
                array,
                is_last=is_last)
        assert self._ops[self._pos - 2] == '\x00'
        assert self._ops[self._pos - 1] == '\x00'
        self._pos -= 2
        self.append_int(s)
        return s

    def create_empty_top_snapshot(self, vable_boxes, vref_boxes):
        self._total_snapshots += 1
        s = len(self._snapshot_data)
        array = self._list_of_boxes([])
        vable_array = self._list_of_boxes_virtualizable(vable_boxes)
        vref_array = self._list_of_boxes(vref_boxes)
        self.append_snapshot_data_int(vable_array)
        self.append_snapshot_data_int(vref_array)
        self._encode_snapshot(
                -1,
                0,
                array,
                is_last=True)
        assert self._ops[self._pos - 2] == '\x00'
        assert self._ops[self._pos - 1] == '\x00'
        self._pos -= 2
        self.append_int(s)
        return s

    def create_snapshot(self, frame, is_last=False):
        self._total_snapshots += 1
        array = frame.get_list_of_active_boxes(True, self.new_array, self._add_box_to_storage)
        self.snapshot_add_prev(SNAPSHOT_PREV_COMES_NEXT)
        return self._encode_snapshot(frame.jitcode.index, frame.pc, array, is_last=is_last)

    def snapshot_add_prev(self, prev):
        assert self._snapshot_data[-2] == '}' # SNAPSHOT_PREV_NEEDS_PATCHING
        assert self._snapshot_data[-1] == '\xff'
        self._snapshot_data.pop()
        self._snapshot_data.pop()
        self.append_snapshot_data_int(prev)

    def capture_resumedata(self, framestack, virtualizable_boxes, virtualref_boxes, after_residual_call=False):
        n = len(framestack) - 1
        if n >= 0:
            top = framestack[n]
            result = self.create_top_snapshot(
                top, virtualizable_boxes,
                virtualref_boxes,
                after_residual_call=after_residual_call,
                is_last=n == 0)
            self._ensure_parent_resumedata(framestack, n)
        else:
            result = self.create_empty_top_snapshot(
                virtualizable_boxes, virtualref_boxes)
        return result

    def _ensure_parent_resumedata(self, framestack, n):
        while n > 0:
            target = framestack[n]
            back = framestack[n - 1]
            if target.parent_snapshot >= 0:
                self.snapshot_add_prev(target.parent_snapshot)
                return
            s = self.create_snapshot(back, is_last=n == 1)
            target.parent_snapshot = s
            n -= 1


    # ____________________________________________________________

    def get_iter(self):
        return TraceIterator(self, self._start, self._pos,
                             metainterp_sd=self.metainterp_sd)

    def get_live_ranges(self):
        t = self.get_iter()
        liveranges = [0] * self._index
        index = t._count
        while not t.done():
            index = t.next_element_update_live_range(index, liveranges)
        return liveranges

    def get_dead_ranges(self):
        """ Same as get_live_ranges, but returns a list of "dying" indexes,
        such as for each index x, the number found there is for sure dead
        before x
        """
        def insert(ranges, pos, v):
            # XXX skiplist
            while ranges[pos]:
                pos += 1
                if pos == len(ranges):
                    return
            ranges[pos] = v

        if self._deadranges != (-1, None):
            if self._deadranges[0] == self._count:
                return self._deadranges[1]
        liveranges = self.get_live_ranges()
        deadranges = [0] * (self._index + 2)
        assert len(deadranges) == len(liveranges) + 2
        for i in range(self._start, len(liveranges)):
            elem = liveranges[i]
            if elem:
                insert(deadranges, elem + 1, i)
        self._deadranges = (self._count, deadranges)
        return deadranges

    def unpack(self):
        iter = self.get_iter()
        ops = []
        try:
            while not iter.done():
                ops.append(iter.next())
        except IndexError:
            pass
        return iter.inputargs, ops

